@@ -7,10 +7,11 @@
 // as published by the Free Software Foundation; either version 2
 // of the License, or (at your option) any later version.
 
-#ifndef _3DSMESH_H_
-#define _3DSMESH_H_
+#ifndef _CELENGINE_3DSMESH_H_
+#define _CELENGINE_3DSMESH_H_
 
 #include <vector>
+#include <string>
 #include <celengine/mesh.h>
 #include <celengine/vertexlist.h>
 #include <cel3ds/3dsmodel.h>
@@ -18,12 +19,13 @@
 class Mesh3DS : public Mesh
 {
  public:
-    Mesh3DS(const M3DScene& scene);
+    Mesh3DS(const M3DScene& scene, const std::string& texturePath);
     ~Mesh3DS();
 
     void render(float lod);
     void render(unsigned int attributes, float lod);
     void render(unsigned int attributes, const Frustum&, float lod);
+    bool pick(const Ray3d& ray, double& distance);
     void normalize();
 
  private:
@@ -31,4 +33,4 @@ class Mesh3DS : public Mesh
     VertexListVec vertexLists;
 };
 
-#endif // _3DSMESH_H_
+#endif // _CELENGINE_3DSMESH_H_
