@@ -192,6 +192,8 @@ static void EclipticToEquatorial(double t, double fEclLat, double fEclLon,
     double eps;
     double deps, dpsi;
 
+    t = (astro::J2000 - 2415020.0) / 36525.0;
+    t = 0;
     eps = Obliquity(t);		// mean obliquity for date
     Nutation(t, deps, dpsi);
     eps += deps;
@@ -212,6 +214,37 @@ static void EclipticToEquatorial(double t, double fEclLat, double fEclLon,
     RA = pfmod(RA, TWOPI);
 }
 
+
+// Convert equatorial coordinates from one epoch to another.  Method is from
+// Chapter 21 of Meeus's _Astronomical Algorithms_
+void EpochConvert(double jdFrom, double jdTo,
+                  double a0, double d0,
+                  double& a, double& d)
+{
+    double T = (jdFrom - astro::J2000) / 36525.0;
+    double t = (jdTo - jdFrom) / 36525.0;
+            
+    double zeta = (2306.2181 + 1.39656 * T - 0.000139 * T * T) * t +
+        (0.30188 - 0.000344 * T) * t * t + 0.017998 * t * t * t;
+    double z = (2306.2181 + 1.39656 * T - 0.000139 * T * T) * t +
+        (1.09468 + 0.000066 * T) * t * t + 0.018203 * t * t * t;
+    double theta = (2004.3109 - 0.85330 * T - 0.000217 * T * T) * t -
+        (0.42665 + 0.000217 * T) * t * t - 0.041833 * t * t * t;
+    zeta  = degToRad(zeta / 3600.0);
+    z     = degToRad(z / 3600.0);
+    theta = degToRad(theta / 3600.0);
+
+    double A = cos(d0) * sin(a0 + zeta);
+    double B = cos(theta) * cos(d0) * cos(a0 + zeta) -
+        sin(theta) * sin(d0);
+    double C = sin(theta) * cos(d0) * cos(a0 + zeta) +
+        cos(theta) * sin(d0);
+
+    a = atan2(A, B) + z;
+    d = asin(C);
+}
+
+
 double meanAnomalySun(double t)
 {
     double t2, a, b;
@@ -529,11 +562,12 @@ class LunarOrbit : public CachingOrbit
 	double ld, ms, md, de, f, n, hp;
 	double a, sa, sn, b, sb, c, sc, e, e2, l, g, w1, w2;
 	double m1, m2, m3, m4, m5, m6;
-    double eclLon, eclLat, horzPar, distance;
-    double RA, dec;
+        double eclLon, eclLat, horzPar, distance;
+        double RA, dec;
 
-    //Computation requires an abbreviated Julian day: epoch January 0.5, 1900.
-    jd19 = jd - 2415020.0;
+        // Computation requires an abbreviated Julian day:
+        // epoch January 0.5, 1900.
+        jd19 = jd - 2415020.0;
 	t = jd19/36525;
 	t2 = t*t;
 
@@ -597,12 +631,12 @@ class LunarOrbit : public CachingOrbit
 	    .001773*sin(md+2*(de-f))-.001595*sin(2*(f+de))+
 	    e*.00122*sin(4*de-ms-md)-.00111*sin(2*(md+f))+.000892*sin(md-3*de);
 	l = l-e*.000811*sin(ms+md+2*de)+e*.000761*sin(4*de-ms-2*md)+
-        e2*.000704*sin(md-2*(ms+de))+e*.000693*sin(ms-2*(md-de))+
-        e*.000598*sin(2*(de-f)-ms)+.00055*sin(md+4*de)+.000538*sin(4*md)+
-        e*.000521*sin(4*de-ms)+.000486*sin(2*md-de);
+            e2*.000704*sin(md-2*(ms+de))+e*.000693*sin(ms-2*(md-de))+
+            e*.000598*sin(2*(de-f)-ms)+.00055*sin(md+4*de)+.000538*sin(4*md)+
+            e*.000521*sin(4*de-ms)+.000486*sin(2*md-de);
 	l = l+e2*.000717*sin(md-2*ms);
-    eclLon = ld+degToRad(l);
-    eclLon = pfmod(eclLon, TWOPI);
+        eclLon = ld+degToRad(l);
+        eclLon = pfmod(eclLon, TWOPI);
 
 	g = 5.12819*sin(f)+.280606*sin(md+f)+.277693*sin(md-f)+
 	    .173238*sin(2*de-f)+.055413*sin(2*de+f-md)+.046272*sin(2*de-f-md)+
@@ -614,9 +648,9 @@ class LunarOrbit : public CachingOrbit
 	    e*.001877*sin(f-ms+md)+.001828*sin(4*de-f-md)-e*.001803*sin(f+ms)-
 	    .00175*sin(3*f);
 	g = g+e*.00157*sin(md-ms-f)-.001487*sin(f+de)-e*.001481*sin(f+ms+md)+
-        e*.001417*sin(f-ms-md)+e*.00135*sin(f-ms)+.00133*sin(f-de)+
-        .001106*sin(f+3*md)+.00102*sin(4*de-f)+.000833*sin(f+4*de-md)+
-        .000781*sin(md-3*f)+.00067*sin(f+4*de-2*md);
+            e*.001417*sin(f-ms-md)+e*.00135*sin(f-ms)+.00133*sin(f-de)+
+            .001106*sin(f+3*md)+.00102*sin(4*de-f)+.000833*sin(f+4*de-md)+
+            .000781*sin(md-3*f)+.00067*sin(f+4*de-2*md);
 	g = g+.000606*sin(2*de-3*f)+.000597*sin(2*(de+md)-f)+
 	    e*.000492*sin(2*de+md-ms-f)+.00045*sin(2*(md-de)-f)+
 	    .000439*sin(3*md-f)+.000423*sin(f+2*(de+md))+
@@ -643,22 +677,28 @@ class LunarOrbit : public CachingOrbit
          e*.000019*cos(4*de-ms-md);
 	horzPar = degToRad(hp);
 
-    //At this point we have values of ecliptic longitude, latitude and horizontal
-    //parallax (eclLong, eclLat, horzPar) in radians.
+        // At this point we have values of ecliptic longitude, latitude and
+        // horizontal parallax (eclLong, eclLat, horzPar) in radians.
 
-    //Now compute distance using horizontal parallax.
-    distance = 6378.14/sin(horzPar);
+        // Now compute distance using horizontal parallax.
+        distance = 6378.14 / sin(horzPar);
 
-    //Finally convert eclLat, eclLon to RA, Dec.
-    EclipticToEquatorial(t, eclLat, eclLon, RA, dec);
+        // Finally convert eclLat, eclLon to RA, Dec.
+        EclipticToEquatorial(t, eclLat, eclLon, RA, dec);
 
-    //Corrections for internal coordinate system
-    dec -= (PI/2);
-    RA += PI;
+        // RA and Dec are referred to the equinox of date; we want to use
+        // the J2000 equinox instead.  A better idea would be to directly 
+        // compute the position of the Moon in this coordinate system, but
+        // this was easier.
+        EpochConvert(jd, astro::J2000, RA, dec, RA, dec);
 
-    return Point3d(cos(RA) * sin(dec) * distance,
-                   cos(dec) * distance,
-                   -sin(RA) * sin(dec) * distance);
+        // Corrections for internal coordinate system
+        dec -= (PI/2);
+        RA += PI;
+
+        return Point3d(cos(RA) * sin(dec) * distance,
+                       cos(dec) * distance,
+                       -sin(RA) * sin(dec) * distance);
     };
 
     double getPeriod() const
@@ -1208,6 +1248,9 @@ class PlutoOrbit : public CachingOrbit
     };
 };
 
+// static const double JupAscendingNode = degToRad(20.453422);
+static const double JupAscendingNode = degToRad(22.203);
+
 class IoOrbit : public CachingOrbit
 {
     Point3d computePosition(double jd) const
@@ -1221,7 +1264,7 @@ class IoOrbit : public CachingOrbit
     double sigma, L, B, R;
     double T, P;
 
-    //Epoch for Galilean satellites is 1976.0 Aug 10
+    // Epoch for Galilean satellites is 1976.0 Aug 10
     t = jd - 2443000.5;
 
     ComputeGalileanElements(t,
@@ -1230,7 +1273,7 @@ class IoOrbit : public CachingOrbit
                             w1, w2, w3, w4,
                             gamma, phi, psi, G, Gp);
 
-    //Calculate periodic terms for longitude
+    // Calculate periodic terms for longitude
     sigma = 0.47259*sin(2*(l1 - l2)) - 0.03478*sin(p3 - p4)
           + 0.01081*sin(l2 - 2*l3 + p3) + 7.38e-3*sin(phi)
           + 7.13e-3*sin(l2 - 2*l3 + p2) - 6.74e-3*sin(p1 + p3 - 2*LPEJ - 2*G)
@@ -1246,15 +1289,14 @@ class IoOrbit : public CachingOrbit
     sigma = degToRad(sigma);
     L = l1 + sigma;
 
-    //Calculate periodic terms for the tangent of the latitude
+    // Calculate periodic terms for the tangent of the latitude
     B = 6.393e-4*sin(L - w1) + 1.825e-4*sin(L - w2)
       + 3.29e-5*sin(L - w3) - 3.11e-5*sin(L - psi)
       + 9.3e-6*sin(L - w4) + 7.5e-6*sin(3*L - 4*l2 - 1.9927*sigma + w2)
       + 4.6e-6*sin(L + psi - 2*LPEJ - 2*G);
-    B = degToRad(B);
-    B = atan(B);    //Not sure this is necessary.
+    B = atan(B);
 
-    //Calculate the periodic terms for distance
+    // Calculate the periodic terms for distance
     R = -4.1339e-3*cos(2*(l1 - l2)) - 3.87e-5*cos(l1 - p3)
       - 2.14e-5*cos(l1 - p4) + 1.7e-5*cos(l1 - l2)
       - 1.31e-5*cos(4*(l1 - l2)) + 1.06e-5*cos(l1 - l3)
@@ -1265,7 +1307,9 @@ class IoOrbit : public CachingOrbit
     P = 1.3966626*T + 3.088e-4*T*T;
     L += degToRad(P);
 
-    //Corrections for internal coordinate system
+    L += JupAscendingNode;
+
+    // Corrections for internal coordinate system
     B -= (PI/2);
     L += PI;
 
@@ -1289,7 +1333,7 @@ class EuropaOrbit : public CachingOrbit
 {
     Point3d computePosition(double jd) const
     {
-    //Computation will yield latitude(L), longitude(B) and distance(R) relative to Jupiter
+    // Computation will yield latitude(L), longitude(B) and distance(R) relative to Jupiter
     double t;
     double l1, l2, l3, l4;
     double p1, p2, p3, p4;
@@ -1298,7 +1342,7 @@ class EuropaOrbit : public CachingOrbit
     double sigma, L, B, R;
     double T, P;
 
-    //Epoch for Galilean satellites is 1976 Aug 10
+    // Epoch for Galilean satellites is 1976 Aug 10
     t = jd - 2443000.5;
 
     ComputeGalileanElements(t,
@@ -1307,7 +1351,7 @@ class EuropaOrbit : public CachingOrbit
                             w1, w2, w3, w4,
                             gamma, phi, psi, G, Gp);
 
-    //Calculate periodic terms for longitude
+    // Calculate periodic terms for longitude
     sigma = 1.06476*sin(2*(l2 - l3)) + 0.04256*sin(l1 - 2*l2 + p3)
           + 0.03581*sin(l2 - p3) + 0.02395*sin(l1 - 2*l2 + p4)
           + 0.01984*sin(l2 - p4) - 0.01778*sin(phi)
@@ -1331,16 +1375,15 @@ class EuropaOrbit : public CachingOrbit
     sigma = degToRad(sigma);
     L = l2 + sigma;
 
-    //Calculate periodic terms for the tangent of the latitude
+    // Calculate periodic terms for the tangent of the latitude
     B = 8.1004e-3*sin(L - w2) + 4.512e-4*sin(L - w3)
       - 3.284e-4*sin(L - psi) + 1.160e-4*sin(L - w4)
       + 2.72e-5*sin(l1 - 2*l3 + 1.0146*sigma + w2) - 1.44e-5*sin(L - w1)
       + 1.43e-5*sin(L + psi - 2*LPEJ - 2*G) + 3.5e-6*sin(L - psi + G)
       - 2.8e-6*sin(l1 - 2*l3 + 1.0146*sigma + w3);
-    B = degToRad(B);
-    B = atan(B);    //Not sure this is necessary.
+    B = atan(B);
 
-    //Calculate the periodic terms for distance
+    // Calculate the periodic terms for distance
     R = 9.3848e-3*cos(l1 - l2) - 3.116e-4*cos(l2 - p3)
       - 1.744e-4*cos(l2 - p4) - 1.442e-4*cos(l2 - p2)
       + 5.53e-5*cos(l2 - l3) + 5.23e-5*cos(l1 - l3)
@@ -1353,7 +1396,9 @@ class EuropaOrbit : public CachingOrbit
     P = 1.3966626*T + 3.088e-4*T*T;
     L += degToRad(P);
 
-    //Corrections for internal coordinate system
+    L += JupAscendingNode;
+
+    // Corrections for internal coordinate system
     B -= (PI/2);
     L += PI;
 
@@ -1364,7 +1409,7 @@ class EuropaOrbit : public CachingOrbit
 
     double getPeriod() const
     {
-        return 3.551810;
+        return 3.5511810791;
     };
 
     double getBoundingRadius() const
@@ -1429,8 +1474,7 @@ class GanymedeOrbit : public CachingOrbit
       - 4.5e-6*sin(L - psi - G) - 4.5e-6*sin(L + psi - 2*LPEJ)
       + 3.7e-6*sin(L + psi - 2*LPEJ - 3*G) + 3.0e-6*sin(2*l2 - 3*L + 4.03*sigma + w2)
       - 2.1e-6*sin(2*l2 - 3*L + 4.03*sigma + w3);
-    B = degToRad(B);
-    B = atan(B);    //Not sure this is necessary.
+    B = atan(B);
 
     //Calculate the periodic terms for distance
     R = -1.4388e-3*cos(l3 - p3) - 7.919e-4*cos(l3 - p4)
@@ -1444,6 +1488,8 @@ class GanymedeOrbit : public CachingOrbit
     P = 1.3966626*T + 3.088e-4*T*T;
     L += degToRad(P);
 
+    L += JupAscendingNode;
+
     //Corrections for internal coordinate system
     B -= (PI/2);
     L += PI;
@@ -1521,8 +1567,7 @@ class CallistoOrbit : public CachingOrbit
       - 5.112e-4*sin(L - w3) + 7.73e-5*sin(L + psi - 2*LPEJ - 2*G)
       + 1.04e-5*sin(L - psi + G) - 1.02e-5*sin(L - psi - G)
       + 8.8e-6*sin(L + psi - 2*LPEJ - 3*G) - 3.8e-6*sin(L + psi - 2*LPEJ - G);
-    B = degToRad(B);
-    B = atan(B);    //Not sure this is necessary.
+    B = atan(B);
 
     //Calculate the periodic terms for distance
     R = -7.3546e-3*cos(l4 - p4) + 1.621e-4*cos(l4 - p3)
@@ -1537,6 +1582,8 @@ class CallistoOrbit : public CachingOrbit
     P = 1.3966626*T + 3.088e-4*T*T;
     L += degToRad(P);
 
+    L += JupAscendingNode;
+
     //Corrections for internal coordinate system
     B -= (PI/2);
     L += PI;
@@ -1558,17 +1605,22 @@ class CallistoOrbit : public CachingOrbit
 };
 
 
-static double sind(double theta)
+static double sinD(double theta)
 {
     return sin(degToRad(theta));
 }
 
-static double cosd(double theta)
+static double cosD(double theta)
 {
     return cos(degToRad(theta));
 }
 
 
+static const double SatAscendingNode = 168.8112;
+static const double SatTilt = 28.0817;
+// static const double SatAscendingNode = 169.530;
+// static const double SatTilt = 28.049;
+
 // Calculations for the orbits of Mimas, Enceladus, Tethys, Dione, Rhea,
 // Titan, Hyperion, and Iapetus are from Jean Meeus's Astronomical Algorithms,
 // and were originally derived by Gerard Dourneau.
@@ -1609,11 +1661,11 @@ void ComputeSaturnianElements(double t,
 static Point3d SaturnMoonPosition(double lam, double gam, double Om, double r)
 {
     double u = lam - Om;
-    double w = Om - 168.8112;
+    double w = Om - SatAscendingNode;
 
     u = degToRad(u);
     w = degToRad(w);
-    gam = degToRad(gam);
+    gam = -degToRad(gam);
     r = r * SaturnRadius;
 
     // Corrections for Celestia's coordinate system
@@ -1633,28 +1685,28 @@ static void OuterSaturnMoonParams(double a, double e, double i,
                                   double& lam, double& gam,
                                   double& r, double& w)
 {
-    double s1 = sind(28.0817);
-    double c1 = cosd(28.0817);
+    double s1 = sinD(SatTilt);
+    double c1 = cosD(SatTilt);
     double e_2 = e * e;
     double e_3 = e_2 * e;
     double e_4 = e_3 * e;
     double e_5 = e_4 * e;
-    double C = (2 * e - 0.25 * e_3 + 0.0520833333 * e_5) * sind(M) +
-        (1.25 * e_2 - 0.458333333 * e_4) * sind(2 * M) +
-        (1.083333333 * e_3 - 0.671875 * e_5) * sind(3 * M) +
-        1.072917 * e_4 * sind(4 * M) + 1.142708 * e_5 * sind(5 * M);
-    double g = Om_ - 168.8112;
-    double a1 = sind(i) * sind(g);
-    double a2 = c1 * sind(i) * cosd(g) - s1 * cosd(i);
+    double C = (2 * e - 0.25 * e_3 + 0.0520833333 * e_5) * sinD(M) +
+        (1.25 * e_2 - 0.458333333 * e_4) * sinD(2 * M) +
+        (1.083333333 * e_3 - 0.671875 * e_5) * sinD(3 * M) +
+        1.072917 * e_4 * sinD(4 * M) + 1.142708 * e_5 * sinD(5 * M);
+    double g = Om_ - SatAscendingNode;
+    double a1 = sinD(i) * sinD(g);
+    double a2 = c1 * sinD(i) * cosD(g) - s1 * cosD(i);
     double u = radToDeg(atan2(a1, a2));
-    double h = c1 * sind(i) - s1 * cosd(i) * cosd(g);
-    double psi = radToDeg(atan2(s1 * sind(g), h));
+    double h = c1 * sinD(i) - s1 * cosD(i) * cosD(g);
+    double psi = radToDeg(atan2(s1 * sinD(g), h));
 
     C = radToDeg(C);
     lam = lam_ + C + u - g - psi;
     gam = radToDeg(asin(sqrt(square(a1) + square(a2))));
-    r = a * (1 - e * e) / (1 + e * cosd(M + C));
-    w = 168.8112 + u;
+    r = a * (1 - e * e) / (1 + e * cosD(M + C));
+    w = SatAscendingNode + u;
 }
 
 
@@ -1671,14 +1723,14 @@ class MimasOrbit : public CachingOrbit
                                  t1, t2, t3, t4, t5, t6, t7, t8, t9, t10, t11,
                                  W0, W1, W2, W3, W4, W5, W6, W7, W8);
 
-        double L = 127.64 + 381.994497 * t1 - 43.57 * sind(W0) -
-            0.720 * sind( 3 * W0) - 0.02144 * sind(5 * W0);
+        double L = 127.64 + 381.994497 * t1 - 43.57 * sinD(W0) -
+            0.720 * sinD( 3 * W0) - 0.02144 * sinD(5 * W0);
         double p = 106.1 + 365.549 * t2;
         double M = L - p;
-        double C = 2.18287 * sind(M) + 0.025988 * sind(2 * M) +
-            0.00043 * sind(3 * M);
+        double C = 2.18287 * sinD(M) + 0.025988 * sinD(2 * M) +
+            0.00043 * sinD(3 * M);
         double lam = L + C;
-        double r = 3.06879 / (1 + 0.01905 * cosd(M + C));
+        double r = 3.06879 / (1 + 0.01905 * cosD(M + C));
         double gam = 1.563;
         double Om = 54.5 - 365.072 * t2;
 
@@ -1710,13 +1762,13 @@ class EnceladusOrbit : public CachingOrbit
                                  t1, t2, t3, t4, t5, t6, t7, t8, t9, t10, t11,
                                  W0, W1, W2, W3, W4, W5, W6, W7, W8);
 
-        double L = 200.317 + 262.7319002 * t1 + 0.25667 * sind(W1) +
-            0.20883 * sind(W2);
+        double L = 200.317 + 262.7319002 * t1 + 0.25667 * sinD(W1) +
+            0.20883 * sinD(W2);
         double p = 309.107 + 123.44121 * t2;
         double M = L - p;
-        double C = 0.55577 * sind(M) + 0.00168 * sind(2 * M);
+        double C = 0.55577 * sinD(M) + 0.00168 * sinD(2 * M);
         double lam = L + C;
-        double r = 3.94118 / (1 + 0.00485 * cosd(M + C));
+        double r = 3.94118 / (1 + 0.00485 * cosD(M + C));
         double gam = 0.0262;
         double Om = 348 - 151.95 * t2;
 
@@ -1748,8 +1800,8 @@ class TethysOrbit : public CachingOrbit
                                  t1, t2, t3, t4, t5, t6, t7, t8, t9, t10, t11,
                                  W0, W1, W2, W3, W4, W5, W6, W7, W8);
 
-        double lam = 285.306 + 190.69791226 * t1 + 2.063 * sind(W0) +
-            0.03409 * sind(3 * W0) + 0.001015 * sind(5 * W0);
+        double lam = 285.306 + 190.69791226 * t1 + 2.063 * sinD(W0) +
+            0.03409 * sinD(3 * W0) + 0.001015 * sinD(5 * W0);
         double r = 4.880998;
         double gam = 1.0976;
         double Om = 111.33 - 72.2441 * t2;
@@ -1782,13 +1834,13 @@ class DioneOrbit : public CachingOrbit
                                  t1, t2, t3, t4, t5, t6, t7, t8, t9, t10, t11,
                                  W0, W1, W2, W3, W4, W5, W6, W7, W8);
 
-        double L = 254.712 + 131.53493193 * t1 - 0.0215 * sind(W1) -
-            0.01733 * sind(W2);
+        double L = 254.712 + 131.53493193 * t1 - 0.0215 * sinD(W1) -
+            0.01733 * sinD(W2);
         double p = 174.8 + 30.820 * t2;
         double M = L - p;
-        double C = 0.24717 * sind(M) + 0.00033 * sind(2 * M);
+        double C = 0.24717 * sinD(M) + 0.00033 * sinD(2 * M);
         double lam = L + C;
-        double r = 6.24871 / (1 + 0.002157 * cosd(M + C));
+        double r = 6.24871 / (1 + 0.002157 * cosD(M + C));
         double gam = 0.0139;
         double Om = 232 - 30.27 * t2;
         // cout << "Dione: " << pfmod(lam, 360.0) << ',' << gam << ',' << pfmod(Om, 360.0) << ',' << r << '\n';
@@ -1823,14 +1875,14 @@ class RheaOrbit : public CachingOrbit
         double e1 = 0.05589 - 0.000346 * t7;
 
         double p_ = 342.7 + 10.057 * t2;
-        double a1 = 0.000265 * sind(p_) + 0.01 * sind(W4);
-        double a2 = 0.000265 * cosd(p_) + 0.01 * cosd(W4);
+        double a1 = 0.000265 * sinD(p_) + 0.01 * sinD(W4);
+        double a2 = 0.000265 * cosD(p_) + 0.01 * cosD(W4);
         double e = sqrt(square(a1) + square(a2));
         double p = radToDeg(atan2(a1, a2));
         double N = 345 - 10.057 * t2;
-        double lam_ = 359.244 + 79.69004720 * t1 + 0.086754 * sind(N);
-        double i = 28.0362 + 0.346898 * cosd(N) + 0.01930 * cosd(W3);
-        double Om = 168.8034 + 0.736936 * sind(N) + 0.041 * sind(W3);
+        double lam_ = 359.244 + 79.69004720 * t1 + 0.086754 * sinD(N);
+        double i = 28.0362 + 0.346898 * cosD(N) + 0.01930 * cosD(W3);
+        double Om = 168.8034 + 0.736936 * sinD(N) + 0.041 * sinD(W3);
         double a = 8.725924;
 
         double lam, gam, r, w;
@@ -1868,11 +1920,11 @@ class TitanOrbit : public CachingOrbit
                                  W0, W1, W2, W3, W4, W5, W6, W7, W8);
         double e1 = 0.05589 - 0.000346 * t7;
 
-        double L = 261.1582 + 22.57697855 * t4 + 0.074025 * sind(W3);
-        double i_ = 27.45141 + 0.295999 * cosd(W3);
-        double Om_ = 168.66925 + 0.628808 * sind(W3);
-        double a1 = sind(W7) * sind(Om_ - W8);
-        double a2 = cosd(W7) * sind(i_) - sind(W7) * cosd(i_) * cosd(Om_ - W8);
+        double L = 261.1582 + 22.57697855 * t4 + 0.074025 * sinD(W3);
+        double i_ = 27.45141 + 0.295999 * cosD(W3);
+        double Om_ = 168.66925 + 0.628808 * sinD(W3);
+        double a1 = sinD(W7) * sinD(Om_ - W8);
+        double a2 = cosD(W7) * sinD(i_) - sinD(W7) * cosD(i_) * cosD(Om_ - W8);
         double g0 = 102.8623;
         double psi = radToDeg(atan2(a1, a2));
         double s = sqrt(square(a1) + square(a2));
@@ -1882,22 +1934,22 @@ class TitanOrbit : public CachingOrbit
         double om;
         for (int n = 0; n < 3; n++)
         {
-            om = W4 + 0.37515 * (sind(2 * g) - sind(2 * g0));
+            om = W4 + 0.37515 * (sinD(2 * g) - sinD(2 * g0));
             g = om - Om_ - psi;
         }
 
-        double e_ = 0.029092 + 0.00019048 * (cosd(2 * g) - cosd(2 * g0));
+        double e_ = 0.029092 + 0.00019048 * (cosD(2 * g) - cosD(2 * g0));
         double q = 2 * (W5 - om);
-        double b1 = sind(i_) * sind(Om_ - W8);
-        double b2 = cosd(W7) * sind(i_) * cosd(Om_ - W8) - sind(W7) * cosd(i_);
+        double b1 = sinD(i_) * sinD(Om_ - W8);
+        double b2 = cosD(W7) * sinD(i_) * cosD(Om_ - W8) - sinD(W7) * cosD(i_);
         double theta = radToDeg(atan2(b1, b2)) + W8;
-        double e = e_ + 0.002778797 * e_ * cosd(q);
-        double p = om + 0.159215 * sind(q);
+        double e = e_ + 0.002778797 * e_ * cosD(q);
+        double p = om + 0.159215 * sinD(q);
         double u = 2 * W5 - 2 * theta + psi;
-        double h = 0.9375 * square(e_) * sind(q) + 0.1875 * square(s) * sind(2 * (W5 - theta));
-        double lam_ = L - 0.254744 * (e1 * sind(W6) + 0.75 * square(e1) * sind(2 * W6) + h);
-        double i = i_ + 0.031843 * s * cosd(u);
-        double Om = Om_ + (0.031843 * s * sind(u)) / sind(i_);
+        double h = 0.9375 * square(e_) * sinD(q) + 0.1875 * square(s) * sinD(2 * (W5 - theta));
+        double lam_ = L - 0.254744 * (e1 * sinD(W6) + 0.75 * square(e1) * sinD(2 * W6) + h);
+        double i = i_ + 0.031843 * s * cosD(u);
+        double Om = Om_ + (0.031843 * s * sinD(u)) / sinD(i_);
         double a = 20.216193;
 
         double lam, gam, r, w;
@@ -1909,7 +1961,7 @@ class TitanOrbit : public CachingOrbit
 
     double getPeriod() const
     {
-        return 15.9669028;
+        return 15.94544758;
     };
 
     double getBoundingRadius() const
@@ -1942,43 +1994,43 @@ class HyperionOrbit : public CachingOrbit
         double phi = 2 * (om - W5);
         double chi = 94.9 - 2.292 * t8;
         double a = 24.50601 -
-            0.08686 * cosd(eta) -
-            0.00166 * cosd(zeta + eta) +
-            0.00175 * cosd(zeta - eta);
+            0.08686 * cosD(eta) -
+            0.00166 * cosD(zeta + eta) +
+            0.00175 * cosD(zeta - eta);
         double e = 0.103458 -
-            0.004099 * cosd(eta) -
-            0.000167 * cosd(zeta + eta) +
-            0.000235 * cosd(zeta - eta) +
-            0.02303 * cosd(zeta) -
-            0.00212 * cosd(2 * zeta) +
-            0.000151 * cosd(3 * zeta) +
-            0.00013 * sind(phi);
+            0.004099 * cosD(eta) -
+            0.000167 * cosD(zeta + eta) +
+            0.000235 * cosD(zeta - eta) +
+            0.02303 * cosD(zeta) -
+            0.00212 * cosD(2 * zeta) +
+            0.000151 * cosD(3 * zeta) +
+            0.00013 * sinD(phi);
         double p = om +
-            0.15648 * sind(chi) -
-            0.4457 * sind(eta) -
-            0.2657 * sind(zeta + eta) -
-            0.3573 * sind(zeta - eta) -
-            12.872 * sind(zeta) +
-            1.668 * sind(2 * zeta) -
-            0.2419 * sind(3 * zeta) - 
-            0.07 * sind(phi);
+            0.15648 * sinD(chi) -
+            0.4457 * sinD(eta) -
+            0.2657 * sinD(zeta + eta) -
+            0.3573 * sinD(zeta - eta) -
+            12.872 * sinD(zeta) +
+            1.668 * sinD(2 * zeta) -
+            0.2419 * sinD(3 * zeta) - 
+            0.07 * sinD(phi);
         double lam_ = 177.047 +
             16.91993829 * t6 +
-            0.15648 * sind(chi) +
-            9.142 * sind(eta) +
-            0.007 * sind(2 * eta) -
-            0.014 * sind(3 * eta) +
-            0.2275 * sind(zeta + eta) +
-            0.2112 * sind(zeta - eta) -
-            0.26 * sind(zeta) -
-            0.0098 * sind(2 * zeta) -
-            0.013 * sind(as) +
-            0.017 * sind(bs) -
-            0.0303 * sind(phi);
-        double i = 27.3347 + 0.643486 * cosd(chi) + 0.315 * cosd(W3) +
-            0.018 * cosd(theta) - 0.018 * cosd(cs);
-        double Om = 168.6812 + 1.40136 * cosd(chi) + 0.68599 * sind(W3) -
-            0.0392 * sind(cs) + 0.0366 * sind(theta_);
+            0.15648 * sinD(chi) +
+            9.142 * sinD(eta) +
+            0.007 * sinD(2 * eta) -
+            0.014 * sinD(3 * eta) +
+            0.2275 * sinD(zeta + eta) +
+            0.2112 * sinD(zeta - eta) -
+            0.26 * sinD(zeta) -
+            0.0098 * sinD(2 * zeta) -
+            0.013 * sinD(as) +
+            0.017 * sinD(bs) -
+            0.0303 * sinD(phi);
+        double i = 27.3347 + 0.643486 * cosD(chi) + 0.315 * cosD(W3) +
+            0.018 * cosD(theta) - 0.018 * cosD(cs);
+        double Om = 168.6812 + 1.40136 * cosD(chi) + 0.68599 * sinD(W3) -
+            0.0392 * sinD(cs) + 0.0366 * sinD(theta_);
 
         double lam, gam, r, w;
         OuterSaturnMoonParams(a, e, i, Om, lam_ - p, lam_,
@@ -2037,37 +2089,37 @@ class IapetusOrbit : public CachingOrbit
         double u4 = lT + gT - g1;
         double u5 = 2 * (ls + gs);
 
-        double a = 58.935028 + 0.004638 * cosd(u1) + 0.058222 * cosd(u2);
+        double a = 58.935028 + 0.004638 * cosD(u1) + 0.058222 * cosD(u2);
         double e = e_ -
-            0.0014097 * cosd(g1 - gT) +
-            0.0003733 * cosd(u5 - 2 * g) +
-            0.0001180 * cosd(u3) +
-            0.0002408 * cosd(l) +
-            0.0002849 * cosd(l + u2) +
-            0.0006190 * cosd(u4);
-        double W = 0.08077 * sind(g1 - gT) +
-            0.02139 * sind(u5 - 2 * g) -
-            0.00676 * sind(u3) +
-            0.01380 * sind(l) +
-            0.01632 * sind(l + u2) +
-            0.03547 * sind(u4);
+            0.0014097 * cosD(g1 - gT) +
+            0.0003733 * cosD(u5 - 2 * g) +
+            0.0001180 * cosD(u3) +
+            0.0002408 * cosD(l) +
+            0.0002849 * cosD(l + u2) +
+            0.0006190 * cosD(u4);
+        double W = 0.08077 * sinD(g1 - gT) +
+            0.02139 * sinD(u5 - 2 * g) -
+            0.00676 * sinD(u3) +
+            0.01380 * sinD(l) +
+            0.01632 * sinD(l + u2) +
+            0.03547 * sinD(u4);
         double p = om0 + W / e_;
         double lam_ = mu - 
-            0.04299 * sind(u2) -
-            0.00789 * sind(u1) -
-            0.06312 * sind(ls) -
-            0.00295 * sind(2 * ls) -
-            0.02231 * sind(u5) +
-            0.00650 * sind(u5 + psi);
+            0.04299 * sinD(u2) -
+            0.00789 * sinD(u1) -
+            0.06312 * sinD(ls) -
+            0.00295 * sinD(2 * ls) -
+            0.02231 * sinD(u5) +
+            0.00650 * sinD(u5 + psi);
         double sum = l + g1 + lT + gT + phi;
         double i = i_ +
-            0.04204 * cosd(u5 + psi) +
-            0.00235 * cosd(sum) +
-            0.00360 * cosd(u2 + phi);
-        double w_ = 0.04204 * sind(u5 + psi) +
-            0.00235 * sind(sum) +
-            0.00358 * sind(u2 + phi);
-        double Om = Om_ + w_ / sind(i_);
+            0.04204 * cosD(u5 + psi) +
+            0.00235 * cosD(sum) +
+            0.00360 * cosD(u2 + phi);
+        double w_ = 0.04204 * sinD(u5 + psi) +
+            0.00235 * sinD(sum) +
+            0.00358 * sinD(u2 + phi);
+        double Om = Om_ + w_ / sinD(i_);
 
         double lam, gam, r, w;
         OuterSaturnMoonParams(a, e, i, Om, lam_ - p, lam_,
@@ -2078,7 +2130,7 @@ class IapetusOrbit : public CachingOrbit
 
     double getPeriod() const
     {
-        return 21.276609;
+        return 79.330183;
     };
 
     double getBoundingRadius() const
@@ -2197,6 +2249,8 @@ public:
         double gamma = 2.0 * asin(sqrt(square(b0) + square(b1)));
         double theta = atan2(b1, b0);
 
+        L += degToRad(174.99);
+
         // Now that we have all the orbital elements, compute the position
         double M = L - p;
 
@@ -2205,6 +2259,7 @@ public:
         double ecc = M;
         for (i = 0; i < 4; i++)
             ecc = M + e * sin(ecc);
+
         double x = a * (cos(ecc) - e);
         double z = a * sqrt(1 - square(e)) * -sin(ecc);
 
@@ -2217,7 +2272,7 @@ public:
 
 
 static double uran_n[5] =
-{ -4.44352267, 2.49254257, 1.51595490, 0.72166316, 0.46658054 };
+{ 4.44352267, 2.49254257, 1.51595490, 0.72166316, 0.46658054 };
 static double uran_a[5] =
 { 129800, 191200, 266000, 435800, 583600 };
 static double uran_L0[5] =
@@ -2304,28 +2359,34 @@ static UranianSatelliteOrbit* CreateUranianSatelliteOrbit(int n)
 };
 
 
-Orbit* GetCustomOrbit(const std::string& name)
+static double yearToJD(int year)
+{
+    return (double) astro::Date(year, 1, 1);
+}
+
+
+Orbit* GetCustomOrbit(const string& name)
 {
     if (name == "mercury")
-        return new MercuryOrbit();
+        return new MixedOrbit(new MercuryOrbit(), yearToJD(-4000), yearToJD(4000), astro::SolarMass);
     if (name == "venus")
-        return new VenusOrbit();
+        return new MixedOrbit(new VenusOrbit(), yearToJD(-4000), yearToJD(4000), astro::SolarMass);
     if (name == "earth")
-        return new EarthOrbit();
+        return new MixedOrbit(new EarthOrbit(), yearToJD(-4000), yearToJD(4000), astro::SolarMass);
     if (name == "moon")
-        return new LunarOrbit();
+        return new MixedOrbit(new LunarOrbit(), yearToJD(-2000), yearToJD(4000), astro::EarthMass + astro::LunarMass);
     if (name == "mars")
-        return new MarsOrbit();
+        return new MixedOrbit(new MarsOrbit(), yearToJD(-4000), yearToJD(4000), astro::SolarMass);
     if (name == "jupiter")
-        return new JupiterOrbit();
+        return new MixedOrbit(new JupiterOrbit(), yearToJD(-4000), yearToJD(4000), astro::SolarMass);
     if (name == "saturn")
-        return new SaturnOrbit();
+        return new MixedOrbit(new SaturnOrbit(), yearToJD(-4000), yearToJD(4000), astro::SolarMass);
     if (name == "uranus")
-        return new UranusOrbit();
+        return new MixedOrbit(new UranusOrbit(), yearToJD(-4000), yearToJD(4000), astro::SolarMass);
     if (name == "neptune")
-        return new NeptuneOrbit();
+        return new MixedOrbit(new NeptuneOrbit(), yearToJD(-4000), yearToJD(4000), astro::SolarMass);
     if (name == "pluto")
-        return new PlutoOrbit();
+        return new MixedOrbit(new PlutoOrbit(), yearToJD(-4000), yearToJD(4000), astro::SolarMass);
     if (name == "io")
         return new IoOrbit();
     if (name == "europa")
