@@ -7,12 +7,16 @@
 // as published by the Free Software Foundation; either version 2
 // of the License, or (at your option) any later version.
 
-#ifndef _LODSPHEREMESH_H_
-#define _LODSPHEREMESH_H_
+#ifndef CELENGINE_LODSPHEREMESH_H_
+#define CELENGINE_LODSPHEREMESH_H_
 
 #include <celmath/vecmath.h>
 #include <celengine/mesh.h>
 #include <celengine/texture.h>
+#include <celengine/glcontext.h>
+
+
+#define MAX_SPHERE_MESH_TEXTURES 4
 
 
 class LODSphereMesh
@@ -21,10 +25,16 @@ public:
     LODSphereMesh();
     ~LODSphereMesh();
 
-    void render(unsigned int attributes, const Frustum&, float lod,
-                Texture* tex);
-    void render(const Frustum&, float lod,
-                Texture* tex);
+    void render(const GLContext&,
+                unsigned int attributes, const Frustum&, float lod,
+                Texture** tex, int nTextures);
+    void render(const GLContext&,
+                unsigned int attributes, const Frustum&, float lod,
+                Texture* tex0 = NULL, Texture* tex1 = NULL,
+                Texture* tex2 = NULL, Texture* tex3 = NULL);
+    void render(const GLContext&,
+                const Frustum&, float lod,
+                Texture** tex, int nTextures);
 
  private:
     int renderPatches(int phi0, int theta0, 
@@ -41,13 +51,14 @@ public:
 
     float* vertices;
     float* normals;
-    float* texCoords;
+    float* texCoords[MAX_SPHERE_MESH_TEXTURES];
     float* tangents;
     int nIndices;
     unsigned short* indices;
 
-    Texture* texture0;
-    unsigned int subtexture0;
+    int nTexturesUsed;
+    Texture* textures[MAX_SPHERE_MESH_TEXTURES];
+    unsigned int subtextures[MAX_SPHERE_MESH_TEXTURES];
 };
 
-#endif // _LODSPHEREMESH_H_
+#endif // CELENGINE_LODSPHEREMESH_H_
