@@ -20,15 +20,23 @@ class MeshInfo : public ResourceInfo<Mesh>
 {
  public:
     std::string source;
+    std::string path;
+    bool resolvedToPath;
 
-    MeshInfo(const std::string _source) : source(_source) {};
+    MeshInfo(const std::string _source,
+             const std::string _path = "") :
+        source(_source), path(_path), resolvedToPath(false) {};
 
+    virtual std::string resolve(const std::string&);
     virtual Mesh* load(const std::string&);
 };
 
 inline bool operator<(const MeshInfo& mi0, const MeshInfo& mi1)
 {
-    return mi0.source < mi1.source;
+    if (mi0.source == mi1.source)
+        return mi0.path < mi1.path;
+    else
+        return mi0.source < mi1.source;
 }
 
 typedef ResourceManager<MeshInfo> MeshManager;
