@@ -8,25 +8,54 @@
 // of the License, or (at your option) any later version.
 
 #include <cassert>
+// #include <limits>
+#include <cstdio>
 
 #ifndef _WIN32
+#ifndef MACOSX_PB
 #include <config.h>
-#endif /* _WIN32 */
+#endif /* ! MACOSX_PB */
+#endif /* ! _WIN32 */
 
 #include <celutil/debug.h>
 #include <celmath/mathlib.h>
+#include <celutil/util.h>
+#include <cstdio>
 #include "astro.h"
 #include "parser.h"
 #include "customorbit.h"
 #include "texmanager.h"
 #include "meshmanager.h"
+#include "trajmanager.h"
 #include "universe.h"
 #include "multitexture.h"
 
 using namespace std;
 
 
-static Surface* CreateSurface(Hash* surfaceData)
+bool getDate(Hash* hash, const string& name, double& jd)
+{
+    // Check first for a number value representing a Julian date
+    if (hash->getNumber(name, jd))
+        return true;
+
+    string dateString;
+    if (hash->getString(name, dateString))
+    {
+        astro::Date date(1, 1, 1);
+        if (astro::parseDate(dateString, date))
+        {
+            jd = (double) date;
+            return true;
+        }
+    }
+
+    return false;
+}
+
+
+static Surface* CreateSurface(Hash* surfaceData,
+                              const std::string& path)
 {
     Surface* surface = new Surface();
 
@@ -46,12 +75,19 @@ static Surface* CreateSurface(Hash* surfaceData)
     string baseTexture;
     string bumpTexture;
     string nightTexture;
+    string specularTexture;
+    string normalTexture;
     bool applyBaseTexture = surfaceData->getString("Texture", baseTexture);
     bool applyBumpMap = surfaceData->getString("BumpMap", bumpTexture);
     bool applyNightMap = surfaceData->getString("NightTexture", nightTexture);
+    bool separateSpecular = surfaceData->getString("SpecularTexture",
+                                                   specularTexture);
+    bool applyNormalMap = surfaceData->getString("NormalMap", normalTexture);
+
     unsigned int baseFlags = Texture::WrapTexture | Texture::AllowSplitting;
     unsigned int bumpFlags = Texture::WrapTexture | Texture::AllowSplitting;
     unsigned int nightFlags = Texture::WrapTexture | Texture::AllowSplitting;
+    unsigned int specularFlags = Texture::WrapTexture | Texture::AllowSplitting;
     
     float bumpHeight = 2.5f;
     surfaceData->getNumber("BumpHeight", bumpHeight);
@@ -73,19 +109,27 @@ static Surface* CreateSurface(Hash* surfaceData)
         surface->appearanceFlags |= Surface::Emissive;
     if (applyBaseTexture)
         surface->appearanceFlags |= Surface::ApplyBaseTexture;
-    if (applyBumpMap)
+    if (applyBumpMap || applyNormalMap)
         surface->appearanceFlags |= Surface::ApplyBumpMap;
     if (applyNightMap)
         surface->appearanceFlags |= Surface::ApplyNightMap;
+    if (separateSpecular)
+        surface->appearanceFlags |= Surface::SeparateSpecularMap;
     if (surface->specularColor != Color(0.0f, 0.0f, 0.0f))
         surface->appearanceFlags |= Surface::SpecularReflection;
 
     if (applyBaseTexture)
-        surface->baseTexture.setTexture(baseTexture, baseFlags);
-    if (applyBumpMap)
-        surface->bumpTexture.setTexture(bumpTexture, bumpHeight, bumpFlags);
+        surface->baseTexture.setTexture(baseTexture, path, baseFlags);
     if (applyNightMap)
-        surface->nightTexture.setTexture(nightTexture, nightFlags);
+        surface->nightTexture.setTexture(nightTexture, path, nightFlags);
+    if (separateSpecular)
+        surface->specularTexture.setTexture(specularTexture, path, specularFlags);
+
+    // If both are present, NormalMap overrides BumpMap
+    if (applyNormalMap)
+        surface->bumpTexture.setTexture(normalTexture, path, bumpFlags);
+    else if (applyBumpMap)
+        surface->bumpTexture.setTexture(bumpTexture, path, bumpHeight, bumpFlags);
 
     return surface;
 }
@@ -132,7 +176,7 @@ static EllipticalOrbit* CreateEllipticalOrbit(Hash* orbitData,
     }
 
     double epoch = astro::J2000;
-    orbitData->getNumber("Epoch", epoch);
+    getDate(orbitData, "Epoch", epoch);
 
     // Accept either the mean anomaly or mean longitude--use mean anomaly
     // if both are specified.
@@ -189,9 +233,13 @@ static RotationElements CreateRotationElements(Hash* rotationData,
     rotationData->getNumber("Obliquity", obliquity);
     re.obliquity = degToRad(obliquity);
 
-    float axisLongitude = 0.0f;
-    rotationData->getNumber("LongOfRotationAxis", axisLongitude);
-    re.axisLongitude = degToRad(axisLongitude);
+    float ascendingNode = 0.0f;
+    rotationData->getNumber("EquatorAscendingNode", ascendingNode);
+    re.ascendingNode = degToRad(ascendingNode);
+
+    float precessionRate = 0.0f;
+    rotationData->getNumber("PrecessionRate", precessionRate);
+    re.precessionRate = degToRad(precessionRate);
 
     return re;
 }
@@ -202,6 +250,7 @@ static RotationElements CreateRotationElements(Hash* rotationData,
 // are in years and AU rather than days and kilometers
 static Body* CreatePlanet(PlanetarySystem* system,
                           Hash* planetData,
+                          const string& path,
                           bool usePlanetUnits = true)
 {
     Body* body = new Body(system);
@@ -217,6 +266,24 @@ static Body* CreatePlanet(PlanetarySystem* system,
                     customOrbitName.c_str());
     }
 
+    if (orbit == NULL)
+    {
+        string sampOrbitFile;
+        if (planetData->getString("SampledOrbit", sampOrbitFile))
+        {
+            DPRINTF(1, "Attempting to load sampled orbit file '%s'\n",
+                    sampOrbitFile.c_str());
+            ResourceHandle orbitHandle =
+                GetTrajectoryManager()->getHandle(TrajectoryInfo(sampOrbitFile, path));
+            orbit = GetTrajectoryManager()->find(orbitHandle);
+            if (orbit == NULL)
+            {
+                DPRINTF(0, "Could not load sampled orbit file '%s'\n",
+                        sampOrbitFile.c_str());
+            }
+        }
+    }
+
     if (orbit == NULL)
     {
         Value* orbitDataValue = planetData->getValue("EllipticalOrbit");
@@ -242,32 +309,87 @@ static Body* CreatePlanet(PlanetarySystem* system,
         return NULL;
     }
     body->setOrbit(orbit);
-    
-    double albedo = 0.5;
-    planetData->getNumber("Albedo", albedo);
-    body->setAlbedo(albedo);
 
     double radius = 10000.0;
     planetData->getNumber("Radius", radius);
     body->setRadius(radius);
 
+    int classification = Body::Unknown;
+    string classificationName;
+    if (planetData->getString("Class", classificationName))
+    {
+        if (compareIgnoringCase(classificationName, "planet") == 0)
+            classification = Body::Planet;
+        else if (compareIgnoringCase(classificationName, "moon") == 0)
+            classification = Body::Moon;
+        else if (compareIgnoringCase(classificationName, "comet") == 0)
+            classification = Body::Comet;
+        else if (compareIgnoringCase(classificationName, "asteroid") == 0)
+            classification = Body::Asteroid;
+        else if (compareIgnoringCase(classificationName, "spacecraft") == 0)
+            classification = Body::Spacecraft;
+        else if (compareIgnoringCase(classificationName, "invisible") == 0)
+            classification = Body::Invisible;
+        else
+            classification = Body::Unknown;
+    }
+
+    if (classification == Body::Unknown)
+    {
+        //Try to guess the type
+        if (system->getPrimaryBody() != NULL)
+        {
+            if(radius > 0.1)
+                classification = Body::Moon;
+            else
+                classification = Body::Spacecraft;
+        }
+        else
+        {
+            if(radius < 1000.0)
+                classification = Body::Asteroid;
+            else
+                classification = Body::Planet;
+        }
+    }
+    body->setClassification(classification);
+
+    // g++ is missing limits header, so we can use this
+    // double beginning   = -numeric_limits<double>::infinity();
+    // double ending      =  numeric_limits<double>::infinity();
+    double beginning   = -1.0e+50;
+    double ending      =  1.0e+50;
+    getDate(planetData, "Beginning", beginning);
+    getDate(planetData, "Ending", ending);
+    body->setLifespan(beginning, ending);
+
+    string infoURL;
+    if (planetData->getString("InfoURL", infoURL))
+        body->setInfoURL(infoURL);
+    
+    double albedo = 0.5;
+    planetData->getNumber("Albedo", albedo);
+    body->setAlbedo(albedo);
+
     double oblateness = 0.0;
     planetData->getNumber("Oblateness", oblateness);
     body->setOblateness(oblateness);
 
+    Quatf orientation;
+    if (planetData->getRotation("Orientation", orientation))
+        body->setOrientation(orientation);
+
     body->setRotationElements(CreateRotationElements(planetData, orbit->getPeriod()));
 
-    Surface* surface = CreateSurface(planetData);
+    Surface* surface = CreateSurface(planetData, path);
     body->setSurface(*surface);
     delete surface;
 
-    
-    
     {
         string mesh("");
         if (planetData->getString("Mesh", mesh))
         {
-            ResourceHandle meshHandle = GetMeshManager()->getHandle(MeshInfo(mesh));
+            ResourceHandle meshHandle = GetMeshManager()->getHandle(MeshInfo(mesh, path));
             body->setMesh(meshHandle);
         }
     }
@@ -299,6 +421,7 @@ static Body* CreatePlanet(PlanetarySystem* system,
                 if (atmosData->getString("CloudMap", cloudTexture))
                 {
                     atmosphere->cloudTexture.setTexture(cloudTexture,
+                                                        path,
                                                         Texture::WrapTexture);
                 }
 
@@ -331,9 +454,10 @@ static Body* CreatePlanet(PlanetarySystem* system,
 
                 string textureName;
                 ringsData->getString("Texture", textureName);
+                MultiResTexture ringTex(textureName, path);
 
                 body->setRings(RingSystem((float) inner, (float) outer,
-                                          color, textureName));
+                                          color, ringTex));
             }
         }
     }
@@ -342,7 +466,9 @@ static Body* CreatePlanet(PlanetarySystem* system,
 }
 
 
-bool LoadSolarSystemObjects(istream& in, Universe& universe)
+bool LoadSolarSystemObjects(istream& in,
+                            Universe& universe,
+                            const std::string& directory)
 {
     Tokenizer tokenizer(&in); 
     Parser parser(&tokenizer);
@@ -392,13 +518,6 @@ bool LoadSolarSystemObjects(istream& in, Universe& universe)
                 solarSystem = universe.createSolarSystem(parent.star);
             }
             parentSystem = solarSystem->getPlanets();
-#ifdef DEBUG
-            if (parentSystem->find(name))
-            {
-                DPRINTF(0, "Warning duplicate definition of %s %s!\n",
-                        parentName.c_str(), name.c_str());
-            }
-#endif // DEBUG
         }
         else if (parent.body != NULL)
         {
@@ -411,16 +530,6 @@ bool LoadSolarSystemObjects(istream& in, Universe& universe)
                 parentSystem = new PlanetarySystem(parent.body);
                 parent.body->setSatellites(parentSystem);
             }
-#ifdef DEBUG
-            else
-            {
-                if (parentSystem->find(name))
-                {
-                    DPRINTF(0, "Warning, duplicate definition of %s %s!\n",
-                            parentName.c_str(), name.c_str());
-                }
-            }
-#endif // DEBUG
             orbitsPlanet = true;
         }
         else
@@ -430,7 +539,13 @@ bool LoadSolarSystemObjects(istream& in, Universe& universe)
 
         if (parentSystem != NULL)
         {
-            Body* body = CreatePlanet(parentSystem, objectData, !orbitsPlanet);
+            if (parentSystem->find(name))
+            {
+                DPRINTF(0, "Warning duplicate definition of %s %s!\n",
+                        parentName.c_str(), name.c_str());
+            }
+
+            Body* body = CreatePlanet(parentSystem, objectData, directory, !orbitsPlanet);
             if (body != NULL)
             {
                 body->setName(name);
@@ -444,9 +559,9 @@ bool LoadSolarSystemObjects(istream& in, Universe& universe)
 }
 
 
-SolarSystem::SolarSystem(const Star* _star) : star(_star)
+SolarSystem::SolarSystem(Star* _star) : star(_star)
 {
-    planets = new PlanetarySystem(star);
+    planets = new PlanetarySystem(_star);
 }
 
 
@@ -467,6 +582,3 @@ PlanetarySystem* SolarSystem::getPlanets() const
 {
     return planets;
 }
-
-
-
