@@ -22,6 +22,7 @@ class Surface
     Surface(Color c = Color(0.0f, 0.0f, 0.0f)) :
         appearanceFlags(0),
         color(c),
+        specularPower(0.0f),
         baseTexture(),
         bumpTexture(),
         nightTexture(),
@@ -37,6 +38,7 @@ class Surface
         ApplySpecularityMap  = 0x20,
         SpecularReflection   = 0x40,
         Emissive             = 0x80,
+        SeparateSpecularMap  = 0x100,
     };
 
     uint32 appearanceFlags;
@@ -47,6 +49,7 @@ class Surface
     MultiResTexture baseTexture;    // surface colors
     MultiResTexture bumpTexture;    // normal map based on terrain relief
     MultiResTexture nightTexture;   // artificial lights to show on night side
+    MultiResTexture specularTexture;// specular mask
     float bumpHeight;               // scale of bump map relief
 };
 
