@@ -7,8 +7,19 @@
 // as published by the Free Software Foundation; either version 2
 // of the License, or (at your option) any later version.
 
+#ifndef MACOSX
 #define IJG_JPEG_SUPPORT
 #define PNG_SUPPORT
+#endif
+
+#ifdef MACOSX
+#include <unistd.h>
+#include "CGBuffer.h"
+#ifndef PNG_SUPPORT
+#include <Quicktime/ImageCompression.h>
+#include <QuickTime/QuickTimeComponents.h>
+#endif
+#endif
 
 #include <cmath>
 #include <algorithm>
@@ -19,14 +30,15 @@
 #include <cassert>
 
 #ifndef _WIN32
+#ifndef MACOSX
 #include <config.h>
-#endif /* _WIN32 */
+#endif /* ! MACOSX */
+#endif /* ! _WIN32 */
 
 #include <celmath/vecmath.h>
 #include <celutil/filetype.h>
 #include <celutil/debug.h>
 
-#define GL_GLEXT_PROTOTYPES
 #include "gl.h"
 #include "glext.h"
 #include "celestia.h"
@@ -50,22 +62,27 @@ extern "C" {
 #include <jpeglib.h>
 #endif
 }
-#endif
 
-#ifdef PNG_SUPPORT
+#endif // IJG_JPEG_SUPPORT
+
+#ifdef PNG_SUPPORT // PNG_SUPPORT
+#ifdef MACOSX
+#include "../../macosx/png.h"
+#else
 #include "png.h"
+#endif // MACOSX
 
 // Define png_jmpbuf() in case we are using a pre-1.0.6 version of libpng
 #ifndef png_jmpbuf
 #define png_jmpbuf(png_ptr) png_ptr->jmpbuf
-#endif
+#endif // PNG_SUPPORT
 
 // Define various expansion transformations for old versions of libpng
 #if PNG_LIBPNG_VER < 10004
 #define png_set_palette_to_rgb(p)  png_set_expand(p)
 #define png_set_gray_1_2_4_to_8(p) png_set_expand(p)
 #define png_set_tRNS_to_alpha(p)   png_set_expand(p)
-#endif
+#endif // PNG_LIBPNG_VER < 10004
 
 #endif // PNG_SUPPORT
 
@@ -101,23 +118,51 @@ typedef struct
 static bool initialized = false;
 static bool compressionSupported = false;
 static bool clampToEdgeSupported = false;
+static bool clampToBorderSupported = false;
 static bool autoMipMapSupported = false;
+static bool maxLevelSupported = false;
 static GLint maxTextureSize = 0;
 
+
+static bool testMaxLevel()
+{
+    unsigned char texels[64];
+
+    glEnable(GL_TEXTURE_2D);
+    // Test whether GL_TEXTURE_MAX_LEVEL is supported . . .
+    glTexImage2D(GL_TEXTURE_2D,
+                 0,
+                 GL_LUMINANCE,
+                 8, 8,
+                 0,
+                 GL_LUMINANCE,
+                 GL_UNSIGNED_BYTE,
+                 texels);
+    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAX_LEVEL, 2);
+    float maxLev = -1.0f;
+    glGetTexParameterfv(GL_TEXTURE_2D, GL_TEXTURE_MAX_LEVEL, &maxLev);
+    glDisable(GL_TEXTURE_2D);
+
+    return maxLev == 2;
+}
+
+
 static void initTextureLoader()
 {
     compressionSupported = ExtensionSupported("GL_ARB_texture_compression");
     if (compressionSupported)
-        InitExtTextureCompression();
+        InitExtension("GL_ARB_texture_compression");
 
 #ifdef GL_VERSION_1_2
     clampToEdgeSupported = true;
 #else
     clampToEdgeSupported = ExtensionSupported("GL_EXT_texture_edge_clamp");
 #endif // GL_VERSION_1_2
+    clampToBorderSupported = ExtensionSupported("GL_ARB_texture_border_clamp");
     autoMipMapSupported = ExtensionSupported("GL_SGIS_generate_mipmap");
+    maxLevelSupported = testMaxLevel();
     glGetIntegerv(GL_MAX_TEXTURE_SIZE, &maxTextureSize);
- 
+
     initialized = true;
 }
 
@@ -206,7 +251,7 @@ Texture::~Texture()
         for (int i = 0; i < uSplit * vSplit; i++)
         {
             if (glNames[i] != 0)
-                glDeleteTextures(1, &glNames[i]);
+                glDeleteTextures(1, (const GLuint*) &glNames[i]);
         }
         delete[] glNames;
     }
@@ -266,7 +311,7 @@ void Texture::bindName(uint32 flags)
     bool compress = ((flags & CompressTexture) != 0) && compressionSupported;
     bool mipmap = ((flags & NoMipMaps) == 0);
     bool automipmap = ((flags & AutoMipMaps) != 0) && autoMipMapSupported && mipmap;
-    if (maxMipMapLevel == 0)
+    if (maxMipMapLevel == 0 || (maxMipMapLevel > 0 && !maxLevelSupported))
         mipmap = false;
 
     if (pixels == NULL)
@@ -285,8 +330,19 @@ void Texture::bindName(uint32 flags)
 
     // If we're not wrapping, use GL_CLAMP_TO_EDGE if it's available; we want
     // to ignore the border color.
-    GLenum wrapMode = wrap ? GL_REPEAT :
-        (clampToEdgeSupported ? GL_CLAMP_TO_EDGE : GL_CLAMP);
+    GLenum wrapMode = GL_REPEAT;
+    if (!wrap)
+    {
+        if ((flags & BorderClamp) != 0 && clampToBorderSupported)
+        {
+            wrapMode = GL_CLAMP_TO_BORDER_ARB;
+        }
+        else
+        {
+            wrapMode = clampToEdgeSupported ? GL_CLAMP_TO_EDGE : GL_CLAMP;
+        }
+    }
+
     if (cubeMap)
     {
         textureType = GL_TEXTURE_CUBE_MAP_EXT;
@@ -318,6 +374,13 @@ void Texture::bindName(uint32 flags)
         case GL_INTENSITY:
             internalFormat = GL_COMPRESSED_INTENSITY_ARB;
             break;
+        case GL_COMPRESSED_RGBA_S3TC_DXT1_EXT:
+        case GL_COMPRESSED_RGBA_S3TC_DXT3_EXT:
+        case GL_COMPRESSED_RGBA_S3TC_DXT5_EXT:
+            compressedFormat = true;
+            internalFormat = format;
+            // compress = false; // Don't recompress DXT textures
+            break;
         }
 	glHint((GLenum) GL_TEXTURE_COMPRESSION_HINT_ARB, GL_NICEST);
     }
@@ -447,6 +510,12 @@ void Texture::bindName(uint32 flags)
             glTexParameteri(textureType, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
             glTexParameteri(textureType, GL_TEXTURE_MIN_FILTER,
                             mipmap ? GL_LINEAR_MIPMAP_LINEAR : GL_LINEAR);
+            if (wrapMode == GL_CLAMP_TO_BORDER_ARB)
+            {
+                float bc[4] = { borderColor.red(), borderColor.green(),
+                                borderColor.blue(), borderColor.alpha() };
+                glTexParameterfv(textureType, GL_TEXTURE_BORDER_COLOR, bc);
+            }
             
             if (automipmap)
                 glTexParameteri(textureType, GL_GENERATE_MIPMAP_SGIS, GL_TRUE);
@@ -523,13 +592,13 @@ void Texture::loadCompressedTexture(unsigned char* texels,
         int mmSize = compressedBlockSize(format) *
             ((mmWidth + 3) / 4) * ((mmHeight + 3) / 4);
                         
-        glCompressedTexImage2DARB(GL_TEXTURE_2D,
-                                  mipMapLevel,
-                                  format,
-                                  mmWidth, mmHeight,
-                                  0,
-                                  mmSize,
-                                  texels + mmOffset);
+        glx::glCompressedTexImage2DARB(GL_TEXTURE_2D,
+                                       mipMapLevel,
+                                       format,
+                                       mmWidth, mmHeight,
+                                       0,
+                                       mmSize,
+                                       texels + mmOffset);
         mmOffset += mmSize;
     }
 }
@@ -611,6 +680,12 @@ void Texture::setMaxMipMapLevel(int level)
 }
 
 
+void Texture::setBorderColor(Color c)
+{
+    borderColor = c;
+}
+
+
 // Convert the texture to a normal map
 void Texture::normalMap(float scale, bool wrap)
 {
@@ -665,7 +740,7 @@ void Texture::normalMap(float scale, bool wrap)
             int h10 = (int) pixels[(i0 * width + j1) * components];
             int h01 = (int) pixels[(i1 * width + j0) * components];
             
-            float dx = (float) (h00 - h10) * (1.0f / 255.0f) * scale;
+            float dx = (float) (h10 - h00) * (1.0f / 255.0f) * scale;
             float dy = (float) (h00 - h01) * (1.0f / 255.0f) * scale;
 
             float mag = (float) sqrt(dx * dx + dy * dy + 1.0f);
@@ -736,24 +811,48 @@ Texture* CreateProceduralTexture(int width, int height,
 }
 
 
+static bool isPow2(int x)
+{
+    return ((x & (x - 1)) == 0);
+}
+
 Texture* LoadTextureFromFile(const string& filename)
 {
     ContentType type = DetermineFileType(filename);
+    Texture* tex = NULL;
 
     switch (type)
     {
     case Content_JPEG:
-        return CreateJPEGTexture(filename.c_str());
+        tex = CreateJPEGTexture(filename.c_str());
+        break;
     case Content_BMP:
-        return CreateBMPTexture(filename.c_str());
+        tex = CreateBMPTexture(filename.c_str());
+        break;
     case Content_PNG:
-        return CreatePNGTexture(filename);
+        tex = CreatePNGTexture(filename);
+        break;
     case Content_DDS:
-        return CreateDDSTexture(filename);
+        tex = CreateDDSTexture(filename);
+        break;
     default:
         DPRINTF(0, "Unrecognized or unsupported image file type.\n");
-        return NULL;
+        break;
     }
+
+    if (tex != NULL)
+    {
+        // Require dimensions of textures to be powers of two.
+        if (!isPow2(tex->width) || !isPow2(tex->height))
+        {
+            DPRINTF(0, "Texture %s has non-power of two dimensions.\n",
+                    filename.c_str());
+            delete tex;
+            tex = NULL;
+        }
+    }
+
+    return tex;
 }
 
 
@@ -917,6 +1016,73 @@ Texture* CreateJPEGTexture(const char* filename,
     // warnings occurred (test whether jerr.pub.num_warnings is nonzero).
 
     return tex;
+    
+#elif MACOSX
+
+    Texture* tex = NULL;
+    CGBuffer* cgJpegImage;
+    size_t img_w, img_h, img_d;
+
+    cgJpegImage = new CGBuffer(filename);
+    if (cgJpegImage == NULL) {
+        char tempcwd[2048];
+        getcwd(tempcwd, sizeof(tempcwd));
+        DPRINTF(0, "CGBuffer :: Error opening JPEG texture file %s/%s\n", tempcwd, filename);
+        delete cgJpegImage;
+        return NULL;
+    }
+
+    if (!cgJpegImage->LoadJPEG()) {
+        char tempcwd[2048];
+        getcwd(tempcwd, sizeof(tempcwd));
+        DPRINTF(0, "CGBuffer :: Error loading JPEG texture file %s/%s\n", tempcwd, filename);
+        delete cgJpegImage;
+        return NULL;
+    }
+    
+    cgJpegImage->Render();
+
+    img_w = (size_t)cgJpegImage->image_size.width;
+    img_h = (size_t)cgJpegImage->image_size.height;
+    img_d = (size_t)((cgJpegImage->image_depth == 8) ? 1 : 4);
+
+    //DPRINTF(0,"cgJpegImage :: %d x %d x %d [%d] bpp\n", img_w, img_h, (size_t)cgJpegImage->image_depth, img_d);
+
+
+#ifdef MACOSX_ALPHA_JPEGS
+    int format = (img_d == 1) ? GL_LUMINANCE : GL_RGBA;
+#else
+    int format = (img_d == 1) ? GL_LUMINANCE : GL_RGB;
+#endif
+    tex = new Texture(img_w, img_h, format);
+    if (tex == NULL || tex->pixels == NULL) {
+        DPRINTF(0, "Could not create Texture\n");
+        delete cgJpegImage;
+        return NULL;
+    }
+    // following code flips image and skips alpha byte if no alpha support
+    unsigned char* bout = (unsigned char*)tex->pixels;
+    unsigned char* bin  = (unsigned char*)cgJpegImage->buffer->data;
+    unsigned int bcount = img_w * img_h * img_d;
+    unsigned int i = 0;
+    bin += bcount+(img_w*img_d); // start one row past end
+    for (i=0; i<bcount; ++i)
+    {
+         // at end of row, move back two rows
+        if ( (i % (img_w * img_d)) == 0 ) bin -= 2*(img_w * img_d);
+
+#ifndef MACOSX_ALPHA_JPEGS
+        if (( (img_d != 1) && !((i&3)^3) )) // skip extra byte 
+        {
+            ++bin;
+        } else 
+#endif // !MACOSX_ALPHA_JPEGS         
+
+        *bout++ = *bin++;
+    }    
+    delete cgJpegImage;
+    return tex;
+    
 #else
     return NULL;
 #endif // IJG_JPEG_SUPPORT
@@ -935,6 +1101,78 @@ Texture* CreatePNGTexture(const string& filename)
 {
 #ifndef PNG_SUPPORT
     return NULL;
+/*
+#if MACOSX
+    FSRef fsr;
+    FSSpec fss;
+    GraphicsImportComponent gi;
+    ImageDescriptionHandle imageDescH = NULL;
+    ImageDescription* desc = NULL;
+    ComponentResult result;
+    Texture* tex = NULL;
+    size_t img_w, img_h, img_d;
+    
+    if (FSPathMakeRef(filename.c_str(), &fsr, false) != noErr) {
+        DPRINTF(0,"CreatePNGTexture :: Could not FSPathMakeRef for %s\n",filename.c_str());
+        return NULL;
+    }
+    
+    if (FSGetCatalogInfo(&fsr, kFSCatInfoNone, NULL, NULL, &fss, NULL) != noErr) {
+        DPRINTF(0,"CreatePNGTexture :: Could not FSRef -> FSSpec\n");
+        return NULL;
+    }
+        
+    if (GetGraphicsImporterForFile(&fss, &gi) != noErr) {
+        DPRINTF(0,"CreatePNGTexture :: Could not GetGraphicsImporterForFile\n");
+        return NULL;
+    }
+    
+    result = GraphicsImportGetImageDescription(gi, &imageDescH);
+    if( noErr != result || imageDescH == NULL ) {
+            DPRINTF(0,"CreatePNGTexture :: Error while retrieving image description\n");
+            return NULL;
+    }
+    
+    desc = *imageDescH;
+    img_w = (size_t)desc->width;
+    img_h = (size_t)desc->height;
+    img_d = (size_t)desc->depth;
+    size_t data_size = img_w * img_h * (img_d >> 3);
+    if( imageDescH != NULL)
+            DisposeHandle((Handle)imageDescH);
+    imageDescH = NULL;
+    desc = NULL;
+
+    GWorldPtr gWorld;
+    QDErr err = noErr;
+    Rect boundsRect = { 0, 0, (short)img_w, (short)img_h };
+
+    MemoryBuffer* buf = MemoryBuffer::Create(data_size);
+    
+    if( buff == NULL ) {
+            error("no bitmap buffer available");
+            exit(1);
+    }
+    
+    err = NewGWorldFromPtr( &gWorld, k32ARGBPixelFormat, &boundsRect, NULL, NULL, 0, 
+                                                    bi->data, bi->bytesPerRow );
+    if (noErr != err) {
+            error("error creating new gworld - %d", err);
+            exit(1);
+    }
+    
+    if( (result = GraphicsImportSetGWorld(gi, gWorld, NULL)) != noErr ) {
+            error("error while setting gworld");
+            exit(1);
+    }
+    
+    if( (result = GraphicsImportDraw(gi)) != noErr ) {
+            error("error while drawing image through qt");
+            exit(1);
+    }
+    
+    DisposeGWorld(gWorld);	
+*/
 #else
     char header[8];
     png_structp png_ptr;
@@ -1065,24 +1303,6 @@ Texture* CreatePNGTexture(const string& filename)
 }
 
 
-#ifdef WORDS_BIGENDIAN
-
-static int readInt(ifstream& in)
-{
-    unsigned char b[4];
-    in.read(reinterpret_cast<char*>(b), 4);
-    return reinterpret_cast<int>(b);
-}
-
-static short readShort(ifstream& in)
-{
-    unsigned char b[2];
-    in.read(reinterpret_cast<char*>(b), 2);
-    return reinterpret_cast<short>(b);
-}
-
-#else
-
 static int readInt(ifstream& in)
 {
     unsigned char b[4];
@@ -1098,7 +1318,6 @@ static short readShort(ifstream& in)
     return ((short) b[1] << 8) + (short) b[0];
 }
 
-#endif
 
 static Texture* CreateBMPTexture(ifstream& in)
 {
@@ -1297,7 +1516,7 @@ Texture* CreateNormalizationCubeMap(int size)
     if (tex == NULL)
         return NULL;
 
-    glGenTextures(1, &tex->glNames[0]);
+    glGenTextures(1, (GLuint*) &tex->glNames[0]);
     glBindTexture(GL_TEXTURE_CUBE_MAP_EXT, tex->glNames[0]);
     
     glTexParameteri(GL_TEXTURE_CUBE_MAP_EXT, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
