@@ -13,21 +13,28 @@
 #include <celmath/vecmath.h>
 #include <celmath/intersect.h>
 #include "astro.h"
+#include "3dsmesh.h"
+#include "meshmanager.h"
 #include "universe.h"
 
 using namespace std;
 
+#define ANGULAR_RES 3.5e-6
 
 Universe::Universe() :
     starCatalog(NULL),
     solarSystemCatalog(NULL),
-    galaxyCatalog(NULL),
-    asterisms(NULL)
+    deepSkyCatalog(NULL),
+    asterisms(NULL),
+    /*boundaries(NULL),*/
+    markers(NULL)
 {
+    markers = new MarkerList();
 }
 
 Universe::~Universe()
 {
+    delete markers;
     // TODO: Clean up!
 }
 
@@ -54,14 +61,14 @@ void Universe::setSolarSystemCatalog(SolarSystemCatalog* catalog)
 }
 
 
-GalaxyList* Universe::getGalaxyCatalog() const
+DeepSkyCatalog* Universe::getDeepSkyCatalog() const
 {
-    return galaxyCatalog;
+    return deepSkyCatalog;
 }
 
-void Universe::setGalaxyCatalog(GalaxyList* catalog)
+void Universe::setDeepSkyCatalog(DeepSkyCatalog* catalog)
 {
-    galaxyCatalog = catalog;
+    deepSkyCatalog = catalog;
 }
 
 
@@ -75,6 +82,15 @@ void Universe::setAsterisms(AsterismList* _asterisms)
     asterisms = _asterisms;
 }
 
+ConstellationBoundaries* Universe::getBoundaries() const
+{
+    return boundaries;
+}
+
+void Universe::setBoundaries(ConstellationBoundaries* _boundaries)
+{
+    boundaries = _boundaries;
+}
 
 // Return the planetary system of a star, or NULL if it has no planets.
 SolarSystem* Universe::getSolarSystem(const Star* star) const
@@ -93,7 +109,7 @@ SolarSystem* Universe::getSolarSystem(const Star* star) const
 
 // Create a new solar system for a star and return a pointer to it; if it
 // already has a solar system, just return a pointer to the existing one.
-SolarSystem* Universe::createSolarSystem(const Star* star) const
+SolarSystem* Universe::createSolarSystem(Star* star) const
 {
     SolarSystem* solarSystem = getSolarSystem(star);
     if (solarSystem != NULL)
@@ -108,6 +124,75 @@ SolarSystem* Universe::createSolarSystem(const Star* star) const
 }
 
 
+MarkerList* Universe::getMarkers() const
+{
+    return markers;
+}
+
+
+void Universe::markObject(const Selection& sel,
+                          float size,
+                          Color color,
+                          Marker::Symbol symbol,
+                          int priority)
+{
+    for (MarkerList::iterator iter = markers->begin();
+         iter != markers->end(); iter++)
+    {
+        if (iter->getObject() == sel)
+        {
+            // Handle the case when the object is already marked.  If the
+            // priority is higher than the existing marker, replace it.
+            // Otherwise, do nothing.
+            if (priority > iter->getPriority())
+            {
+                markers->erase(iter);
+                break;
+            }
+            else
+            {
+                return;
+            }
+        }
+    }
+
+    Marker marker(sel);
+    marker.setColor(color);
+    marker.setSymbol(symbol);
+    marker.setSize(size);
+    marker.setPriority(priority);
+    markers->insert(markers->end(), marker);
+}
+
+
+void Universe::unmarkObject(const Selection& sel, int priority)
+{
+    for (MarkerList::iterator iter = markers->begin();
+         iter != markers->end(); iter++)
+    {
+        if (iter->getObject() == sel)
+        {
+            if (priority >= iter->getPriority())
+                markers->erase(iter);
+            break;
+        }
+    }
+}
+
+
+bool Universe::isMarked(const Selection& sel, int priority) const
+{
+    for (MarkerList::iterator iter = markers->begin();
+         iter != markers->end(); iter++)
+    {
+        if (iter->getObject() == sel)
+            return iter->getPriority() >= priority;
+    }
+
+    return false;
+}
+
+
 class ClosestStarFinder : public StarHandler
 {
 public:
@@ -138,8 +223,9 @@ void ClosestStarFinder::process(const Star& star, float distance, float appMag)
 
 struct PlanetPickInfo
 {
-    float cosClosestAngle;
+    double sinAngle2Closest;
     double closestDistance;
+    double closestApproxDistance;
     Body* closestBody;
     Ray3d pickRay;
     double jd;
@@ -150,23 +236,34 @@ static bool ApproxPlanetPickTraversal(Body* body, void* info)
 {
     PlanetPickInfo* pickInfo = (PlanetPickInfo*) info;
 
+    // Reject invisible bodies
+    if (body->getClassification() == Body::Invisible)
+        return true;
+
     Point3d bpos = body->getHeliocentricPosition(pickInfo->jd);
     Vec3d bodyDir = bpos - pickInfo->pickRay.origin;
+    double distance = bodyDir.length();
 
     // Check the apparent radius of the orbit against our tolerance factor.
     // This check exists to make sure than when picking a distant, we select
     // the planet rather than one of its satellites.
     float appOrbitRadius = (float) (body->getOrbit()->getBoundingRadius() /
-                                    bodyDir.length());
-    if (pickInfo->atanTolerance > appOrbitRadius)
+                                    distance);
+    
+    if ((pickInfo->atanTolerance > ANGULAR_RES ? pickInfo->atanTolerance: 
+        ANGULAR_RES) > appOrbitRadius)
         return true;
 
     bodyDir.normalize();
-    double cosAngle = bodyDir * pickInfo->pickRay.direction;
-    if (cosAngle > pickInfo->cosClosestAngle)
+    Vec3d bodyMiss = bodyDir - pickInfo->pickRay.direction;
+    double sinAngle2 = sqrt(bodyMiss * bodyMiss)/2.0;
+
+    if (sinAngle2 <= pickInfo->sinAngle2Closest)
     {
-        pickInfo->cosClosestAngle = cosAngle;
+        pickInfo->sinAngle2Closest = sinAngle2 > ANGULAR_RES ? sinAngle2 : 
+	                                         ANGULAR_RES;
         pickInfo->closestBody = body;
+        pickInfo->closestApproxDistance = distance;
     }
 
     return true;
@@ -182,7 +279,8 @@ static bool ExactPlanetPickTraversal(Body* body, void* info)
     double distance = -1.0;
 
     // Test for intersection with the bounding sphere
-    if (testIntersection(pickInfo->pickRay, Sphered(bpos, radius), distance))
+    if (body->getClassification() != Body::Invisible &&
+        testIntersection(pickInfo->pickRay, Sphered(bpos, radius), distance))
     {
         if (body->getMesh() == InvalidResource)
         {
@@ -197,7 +295,7 @@ static bool ExactPlanetPickTraversal(Body* body, void* info)
                                     radius * (1.0 - body->getOblateness()),
                                     radius);
                 // Transform rotate the pick ray into object coordinates
-                Mat3d m = conjugate(body->getEclipticalToEquatorial()).toMatrix3();
+                Mat3d m = conjugate(body->getEclipticalToEquatorial(pickInfo->jd)).toMatrix3();
                 Ray3d r(Point3d(0, 0, 0) + (pickInfo->pickRay.origin - bpos),
                         pickInfo->pickRay.direction);
                 r = r * m;
@@ -207,11 +305,41 @@ static bool ExactPlanetPickTraversal(Body* body, void* info)
         }
         else
         {
-            // TODO: Check for intersection with the object mesh.  For now,
-            // we just use the bounding sphere.
-        }
+            // Transform rotate the pick ray into object coordinates
+            Quatf qf = body->getOrientation();
+            Quatd qd(qf.w, qf.x, qf.y, qf.z);
+            Mat3d m = conjugate(qd * body->getEclipticalToGeographic(pickInfo->jd)).toMatrix3();
+            Ray3d r(Point3d(0, 0, 0) + (pickInfo->pickRay.origin - bpos),
+                    pickInfo->pickRay.direction);
+            r = r * m;
 
-        if (distance > 0.0 && distance < pickInfo->closestDistance)
+            // The mesh vertices are normalized, then multiplied by a scale
+            // factor.  Thus, the ray needs to be multiplied by the inverse of
+            // the mesh scale factor.
+            double s = 1.0 / radius;
+            r.origin.x *= s;
+            r.origin.y *= s;
+            r.origin.z *= s;
+            r.direction *= s;
+
+            Mesh* mesh = GetMeshManager()->find(body->getMesh());
+            if (mesh != NULL)
+            {
+                if (!mesh->pick(r, distance))
+                    distance = -1.0;
+            }
+        }
+        // Make also sure that the pickRay does not intersect the body in the
+        // opposite hemisphere! Hence, need again the "bodyMiss" angle
+
+        Vec3d bodyDir = bpos - pickInfo->pickRay.origin;
+        bodyDir.normalize();
+        Vec3d bodyMiss = bodyDir - pickInfo->pickRay.direction;
+        double sinAngle2 = sqrt(bodyMiss * bodyMiss)/2.0;
+
+
+        if (sinAngle2 < sin(PI/4.0) && distance > 0.0 && 
+            distance  <= pickInfo->closestDistance)
         {
             pickInfo->closestDistance = distance;
             pickInfo->closestBody = body;
@@ -229,33 +357,61 @@ Selection Universe::pickPlanet(SolarSystem& solarSystem,
                                float faintestMag,
                                float tolerance)
 {
+    double sinTol2 = (sin(tolerance/2.0) >  ANGULAR_RES ? 
+	              sin(tolerance/2.0) : ANGULAR_RES);
     PlanetPickInfo pickInfo;
 
     // Transform the pick direction
-    pickInfo.pickRay = Ray3d(astro::heliocentricPosition(origin, solarSystem.getCenter()), Vec3d(direction.x, direction.y, direction.z));
+    pickInfo.pickRay = Ray3d(astro::heliocentricPosition(origin, 
+    solarSystem.getCenter()), Vec3d(direction.x, direction.y, direction.z));
                              
-    pickInfo.cosClosestAngle = -1.0f;
+    pickInfo.sinAngle2Closest = 1.0;
     pickInfo.closestDistance = 1.0e50;
+    pickInfo.closestApproxDistance = 1.0e50;
     pickInfo.closestBody = NULL;
     pickInfo.jd = when;
     pickInfo.atanTolerance = (float) atan(tolerance);
 
-    // First see if there's a planet that the pick ray intersects.
-    // Select the closest planet intersected.
+    // First see if there's a planet|moon that the pick ray intersects.
+    // Select the closest planet|moon intersected.
+
     solarSystem.getPlanets()->traverse(ExactPlanetPickTraversal,
                                        (void*) &pickInfo);
-    if (pickInfo.closestBody != NULL)
-        return Selection(pickInfo.closestBody);
 
-    // If no planet was intersected by the pick ray, choose the planet
+    if (pickInfo.closestBody != NULL)
+    {
+        // Retain that body
+        Body* closestBody = pickInfo.closestBody;
+
+	// Check if there is a satellite in front of the primary body that is
+	// sufficiently close to the pickRay 
+
+	solarSystem.getPlanets()->traverse(ApproxPlanetPickTraversal,
+					 (void*) &pickInfo);
+        if (pickInfo.closestBody == closestBody)
+	  return  Selection(closestBody); 
+        // Nothing else around, select the body and return
+
+        // Are we close enough to the satellite and is it in front of the body?
+	if ((pickInfo.sinAngle2Closest <= sinTol2) && 
+            (pickInfo.closestDistance > pickInfo.closestApproxDistance)) 
+        return Selection(pickInfo.closestBody);
+            // Yes, select the satellite
+	else
+	    return  Selection(closestBody); 
+           //  No, select the primary body 
+    }
+
+    // If no planet was intersected by the pick ray, choose the planet|moon
     // with the smallest angular separation from the pick ray.  Very distant
     // planets are likley to fail the intersection test even if the user
     // clicks on a pixel where the planet's disc has been rendered--in order
     // to make distant planets visible on the screen at all, their apparent
     // size has to be greater than their actual disc size.
+
     solarSystem.getPlanets()->traverse(ApproxPlanetPickTraversal,
                                        (void*) &pickInfo);
-    if (pickInfo.cosClosestAngle > cos(tolerance))
+    if (pickInfo.sinAngle2Closest <= sinTol2)
         return Selection(pickInfo.closestBody);
     else
         return Selection();
@@ -275,7 +431,7 @@ public:
     const Star* pickedStar;
     Point3f pickOrigin;
     Vec3f pickRay;
-    float cosAngleClosest;
+    double sinAngle2Closest;
 };
 
 StarPicker::StarPicker(const Point3f& _pickOrigin, const Vec3f& _pickRay,
@@ -283,7 +439,8 @@ StarPicker::StarPicker(const Point3f& _pickOrigin, const Vec3f& _pickRay,
     pickedStar(NULL),
     pickOrigin(_pickOrigin),
     pickRay(_pickRay),
-    cosAngleClosest((float) cos(angle))
+    sinAngle2Closest(sin(angle/2.0) > ANGULAR_RES ? sin(angle/2.0) : 
+                                      ANGULAR_RES )
 {
 }
 
@@ -292,10 +449,14 @@ void StarPicker::process(const Star& star, float distance, float appMag)
     Vec3f starDir = star.getPosition() - pickOrigin;
     starDir.normalize();
 
-    float cosAngle = starDir * pickRay;
-    if (cosAngle > cosAngleClosest)
+    Vec3f starMiss = starDir - pickRay;
+    Vec3d sMd = Vec3d(starMiss.x, starMiss.y, starMiss.z); 
+    
+    double sinAngle2 = sqrt(sMd * sMd)/2.0;
+
+    if (sinAngle2 <= sinAngle2Closest)
     {
-        cosAngleClosest = cosAngle;
+        sinAngle2Closest = sinAngle2 > ANGULAR_RES ? sinAngle2 : ANGULAR_RES;
         pickedStar = &star;
     }
 }
@@ -317,7 +478,7 @@ public:
     float maxDistance;
     const Star* closestStar;
     float closestDistance;
-    float cosAngleClosest;
+    double sinAngle2Closest;
 };
 
 
@@ -330,7 +491,8 @@ CloseStarPicker::CloseStarPicker(const UniversalCoord& pos,
     maxDistance(_maxDistance),
     closestStar(NULL),
     closestDistance(0.0f),
-    cosAngleClosest((float) cos(angle))
+    sinAngle2Closest(sin(angle/2.0) > ANGULAR_RES ? sin(angle/2.0) : 
+                                      ANGULAR_RES )
 {
 }
 
@@ -341,11 +503,11 @@ void CloseStarPicker::process(const Star& star,
     if (lowPrecDistance > maxDistance)
         return;
 
-    Vec3f starDir = (star.getPosition() - pickOrigin) *
-        astro::lightYearsToKilometers(1.0f);
+    Point3d hPos = astro::heliocentricPosition(pickOrigin, star.getPosition());
+    Vec3f starDir((float) -hPos.x, (float) -hPos.y, (float) -hPos.z);
     float distance = 0.0f;
 
-    if (testIntersection(Ray3f(Point3f(0, 0, 0), pickDir),
+     if (testIntersection(Ray3f(Point3f(0, 0, 0), pickDir),
                          Spheref(Point3f(starDir.x, starDir.y, starDir.z),
                                  star.getRadius()), distance))
     {
@@ -355,7 +517,8 @@ void CloseStarPicker::process(const Star& star,
             {
                 closestStar = &star;
                 closestDistance = starDir.length();
-                cosAngleClosest = 1.0f; // An exact hit--set the angle to zero
+                sinAngle2Closest = ANGULAR_RES; 
+                // An exact hit--set the angle to "zero"
             }
         }
     }
@@ -363,14 +526,18 @@ void CloseStarPicker::process(const Star& star,
     {
         // We don't have an exact hit; check to see if we're close enough
         float distance = starDir.length();
-        starDir *= (1.0f / distance);
-        float cosAngle = starDir * pickDir;
-        if (cosAngle > cosAngleClosest &&
+        starDir.normalize();
+        Vec3f starMiss = starDir - pickDir;
+        Vec3d sMd = Vec3d(starMiss.x, starMiss.y, starMiss.z ); 
+    
+        double sinAngle2 = sqrt(sMd * sMd)/2.0;
+
+        if (sinAngle2 <= sinAngle2Closest &&
             (closestStar == NULL || distance < closestDistance))
         {
             closestStar = &star;
             closestDistance = distance;
-            cosAngleClosest = cosAngle;
+            sinAngle2Closest = sinAngle2 > ANGULAR_RES ? sinAngle2 : ANGULAR_RES;
         }
     }
 }
@@ -381,6 +548,11 @@ Selection Universe::pickStar(const UniversalCoord& origin,
                              float faintestMag,
                              float tolerance)
 {
+    Point3f o = (Point3f) origin;
+    o.x *= 1e-6f;
+    o.y *= 1e-6f;
+    o.z *= 1e-6f;
+
     // Use a high precision pick test for any stars that are close to the
     // observer.  If this test fails, use a low precision pick test for stars
     // which are further away.  All this work is necessary because the low
@@ -388,7 +560,7 @@ Selection Universe::pickStar(const UniversalCoord& origin,
     // precision test isn't nearly fast enough to use on our database of
     // over 100k stars.
     CloseStarPicker closePicker(origin, direction, 1.0f, tolerance);
-    starCatalog->findCloseStars(closePicker, (Point3f) origin, 1.0f);
+    starCatalog->findCloseStars(closePicker, o, 1.0f);
     if (closePicker.closestStar != NULL)
         return Selection(const_cast<Star*>(closePicker.closestStar));
 
@@ -397,13 +569,15 @@ Selection Universe::pickStar(const UniversalCoord& origin,
     // the rotation required to map (0, 0, -1) to the direction.
     Quatf rotation;
     Vec3f n(0, 0, -1);
-    float epsilon = 1.0e-6f;
-    float cosAngle = n * direction;
-    if (cosAngle > 1.0f - epsilon)
+    Vec3f Missf = n - direction;
+    Vec3d Miss = Vec3d(Missf.x, Missf.y, Missf.z); 
+    double sinAngle2 = sqrt(Miss * Miss)/2.0;
+
+    if (sinAngle2 <= ANGULAR_RES)
     {
         rotation.setAxisAngle(Vec3f(1, 0, 0), 0);
     }
-    else if (cosAngle < epsilon - 1.0f)
+    else if (sinAngle2 >= 1.0 - 0.5 * ANGULAR_RES * ANGULAR_RES)
     {
         rotation.setAxisAngle(Vec3f(1, 0, 0), (float) PI);
     }
@@ -411,12 +585,11 @@ Selection Universe::pickStar(const UniversalCoord& origin,
     {
         Vec3f axis = direction ^ n;
         axis.normalize();
-        rotation.setAxisAngle(axis, (float) acos(cosAngle));
+        rotation.setAxisAngle(axis, (float) 2.0 * asin(sinAngle2));
     }
-    
-    StarPicker picker((Point3f) origin, direction, tolerance);
+    StarPicker picker(o, direction, tolerance);
     starCatalog->findVisibleStars(picker,
-                                  (Point3f) origin,
+                                  o,
                                   rotation,
                                   tolerance, 1.0f,
                                   faintestMag);
@@ -427,6 +600,42 @@ Selection Universe::pickStar(const UniversalCoord& origin,
 }
 
 
+Selection Universe::pickDeepSkyObject(const UniversalCoord& origin,
+                                      const Vec3f& direction,
+                                      float faintestMag,
+                                      float tolerance)
+{
+    Selection sel;
+    Point3d p = (Point3d) origin;
+    Ray3d pickRay(Point3d(p.x * 1e-6, p.y * 1e-6, p.z * 1e-6),
+                  Vec3d(direction.x, direction.y, direction.z));
+    double closestDistance = 1.0e30;
+
+    if (deepSkyCatalog != NULL)
+    {
+        for (DeepSkyCatalog::const_iterator iter = deepSkyCatalog->begin();
+             iter != deepSkyCatalog->end(); iter++)
+        {
+            double distance = 0.0;
+            if (testIntersection(pickRay, 
+                                 Sphered((*iter)->getPosition(), (*iter)->getRadius()),
+                                 distance))
+            {
+                // Don't select an object that contains the origin
+                if (pickRay.origin.distanceTo((*iter)->getPosition()) > (*iter)->getRadius() &&
+                    distance < closestDistance)
+                {
+                    closestDistance = distance;
+                    sel = Selection(*iter);
+                }
+            }
+        }
+    }
+
+    return sel;
+}
+
+
 Selection Universe::pick(const UniversalCoord& origin,
                          const Vec3f& direction,
                          double when,
@@ -448,13 +657,16 @@ Selection Universe::pick(const UniversalCoord& origin,
     if (sel.empty())
         sel = pickStar(origin, direction, faintestMag, tolerance);
 
+    if (sel.empty())
+        sel = pickDeepSkyObject(origin, direction, faintestMag, tolerance);
+
     return sel;
 }
 
 
 // Select an object by name, with the following priority:
 //   1. Try to look up the name in the star catalog
-//   2. Search the galaxy catalog for a matching name.
+//   2. Search the deep sky catalog for a matching name.
 //   3. Check the solar systems for planet names; we don't make any decisions
 //      about which solar systems are relevant, and let the caller pass them
 //      to us to search.
@@ -462,12 +674,12 @@ Selection Universe::find(const string& s,
                          PlanetarySystem** solarSystems,
                          int nSolarSystems)
 {
-    if (galaxyCatalog != NULL)
+    if (deepSkyCatalog != NULL)
     {
-        for (GalaxyList::const_iterator iter = galaxyCatalog->begin();
-             iter != galaxyCatalog->end(); iter++)
+        for (DeepSkyCatalog::const_iterator iter = deepSkyCatalog->begin();
+             iter != deepSkyCatalog->end(); iter++)
         {
-            if ((*iter)->getName() == s)
+            if (compareIgnoringCase((*iter)->getName(), s) == 0)
                 return Selection(*iter);
         }
     }
@@ -510,8 +722,7 @@ Selection Universe::findPath(const string& s,
     if (sel.empty())
         return sel;
 
-    // Don't support paths relative to a galaxy . . . for now.
-    if (sel.galaxy != NULL)
+    if (sel.deepsky != NULL)
         return Selection();
 
     const PlanetarySystem* worlds = NULL;
@@ -561,7 +772,9 @@ Selection Universe::findPath(const string& s,
 // with in one light year.
 SolarSystem* Universe::getNearestSolarSystem(const UniversalCoord& position) const
 {
+    Point3f pos = (Point3f) position;
+    Point3f lyPos(pos.x * 1.0e-6f, pos.y * 1.0e-6f, pos.z * 1.0e-6f);
     ClosestStarFinder closestFinder(1.0f);
-    starCatalog->findCloseStars(closestFinder, (Point3f) position, 1.0f);
+    starCatalog->findCloseStars(closestFinder, lyPos, 1.0f);
     return getSolarSystem(closestFinder.closestStar);
 }
