@@ -19,10 +19,11 @@ struct CelestiaConfig
     std::string starNamesFile;
     std::vector<std::string> catalogXrefFiles;
     std::vector<std::string> solarSystemFiles;
-    std::string extrasDir;
-    std::string galaxyCatalog;
+    std::vector<std::string> extrasDirs;
+    std::string deepSkyCatalog;
     std::vector<std::string> labelledStars;
     std::string asterismsFile;
+    std::string boundariesFile;
     float faintestVisible;
     std::string favoritesFile;
     std::string initScriptFile;
@@ -32,6 +33,8 @@ struct CelestiaConfig
     std::string labelFont;
     std::string titleFont;
     std::string logoTextureFile;
+    std::vector<std::string> ignoreGLExtensions;
+    float rotateAcceleration;
 };
 
 
