@@ -27,6 +27,15 @@
 #define IDD_ADDLOCATION_FOLDER          128
 #define IDD_ORGANIZE_LOCATIONS          129
 #define IDD_RENAME_LOCATION             130
+#define IDD_COMPUTEECLIPSES             131
+#define IDD_ECLIPSEFINDER               132
+#define IDB_FOLDERCLOSED                532
+#define IDB_FOLDEROPENED                534
+#define IDB_LOCATION                    536
+#define IDB_CAMERA                      537
+#define IDB_CAMCORDER                   538
+#define IDB_SCRIPT                      539
+#define IDB_CLOCK                       540
 #define IDC_FINDOBJECT_EDIT             1001
 #define IDC_LICENSE_TEXT                1004
 #define IDC_GLINFO_TEXT                 1005
@@ -68,8 +77,8 @@
 #define IDC_SHOWPLANETS                 1041
 #define IDC_SHOWSTARS                   1042
 #define IDC_SHOWECLIPSESHADOWS          1043
-#define IDC_LABELMAJORPLANETS           1044
-#define IDC_LABELMINORPLANETS           1045
+#define IDC_LABELPLANETS                1044
+#define IDC_LABELMOONS                  1045
 #define IDC_LABELSTARS                  1046
 #define IDC_LABELCONSTELLATIONS         1047
 #define IDC_LABELGALAXIES               1048
@@ -86,6 +95,22 @@
 #define IDC_ORGANIZE_LOCATIONS_DELETE   1061
 #define IDC_NEWLOCATION                 1062
 #define IDC_SHOWSTARSASPOINTS           1063
+#define IDC_LABELASTEROIDS              1064
+#define IDC_LABELSPACECRAFT             1065
+#define IDC_ECLIPSES_LIST               1071
+#define IDCOMPUTE                       1078
+#define IDC_DATEFROM                    1079
+#define IDC_ECLIPSEFROM                 1079
+#define IDC_DATETO                      1080
+#define IDC_ECLIPSETO                   1080
+#define IDSETDATEANDGO                  1081
+#define IDC_EDIT_FILTER_DISTANCE        1084
+#define IDC_SLIDER_FILTER_DISTANCE      1085
+#define IDC_SHOWCOMETTAILS              1086
+#define IDC_SHOWCONSTELLATIONBORDERS    1087
+#define IDC_SHOWRINGSHADOWS             1088
+#define IDC_ECLIPSETYPE                 1089
+#define IDC_ECLIPSETARGET               1090
 #define ID_SELECTSOL                    40001
 #define ID_GOTOSELECTION                40002
 #define ID_FOLLOWSELECTION              40003
@@ -104,7 +129,7 @@
 #define ID_NAVIGATION_GOTO              40020
 #define ID_NAVIGATION_FOLLOW            40021
 #define ID_NAVIGATION_TOURGUIDE         40022
-#define ID_NAVIGATION_BLAH              40023
+#define ID_NAVIGATION_ECLIPSEFINDER     40023
 #define ID_TIME_FASTER                  40024
 #define ID_TIME_SLOWER                  40025
 #define ID_TIME_FREEZE                  40026
@@ -134,15 +159,24 @@
 #define ID_RENDER_VIEWOPTIONS           40061
 #define ID_RENDER_ANTIALIASING          40062
 #define ID_LOCATIONS_ORGANIZE           40063
+#define ID_FILE_OPENSCRIPT              40064
+#define ID_TOOLS_MARK                   40065
+#define ID_TOOLS_UNMARK                 40066
+#define ID_VIEW_HSPLIT                  40067
+#define ID_VIEW_VSPLIT                  40068
+#define ID_VIEW_DELETE_ACTIVE           40069
+#define ID_VIEW_SINGLE                  40070
+#define ID_VIEW_SHOW_FRAMES             40071
+#define ID_VIEW_SYNC_TIME               40072
 #define ID_LOCATIONS_FIRSTLOCATION      41000
 
 // Next default values for new objects
 // 
 #ifdef APSTUDIO_INVOKED
 #ifndef APSTUDIO_READONLY_SYMBOLS
-#define _APS_NEXT_RESOURCE_VALUE        132
-#define _APS_NEXT_COMMAND_VALUE         40064
-#define _APS_NEXT_CONTROL_VALUE         1064
+#define _APS_NEXT_RESOURCE_VALUE        134
+#define _APS_NEXT_COMMAND_VALUE         40065
+#define _APS_NEXT_CONTROL_VALUE         1091
 #define _APS_NEXT_SYMED_VALUE           101
 #endif
 #endif
