@@ -43,7 +43,7 @@ HTREEITEM PopulateLocationsTree(HWND hTree, CelestiaCore* appCore, HINSTANCE app
     // Associate the image list with the tree-view control.
     TreeView_SetImageList(hTree, himlIcons, TVSIL_NORMAL);
 
-    const FavoritesList* favorites = appCore->getFavorites();
+    FavoritesList* favorites = appCore->getFavorites();
     if (favorites != NULL)
     {
         //Create a subtree item called "Locations"
@@ -58,7 +58,7 @@ HTREEITEM PopulateLocationsTree(HWND hTree, CelestiaCore* appCore, HINSTANCE app
         tvis.item.iSelectedImage = 2;
         if (hParent = TreeView_InsertItem(hTree, &tvis))
         {
-            FavoritesList::const_iterator iter = favorites->begin();
+            FavoritesList::iterator iter = favorites->begin();
             while (iter != favorites->end())
             {
                 TVINSERTSTRUCT tvis;
@@ -77,7 +77,7 @@ HTREEITEM PopulateLocationsTree(HWND hTree, CelestiaCore* appCore, HINSTANCE app
 
                     if (hParentItem = TreeView_InsertItem(hTree, &tvis))
                     {
-                        FavoritesList::const_iterator subIter = favorites->begin();
+                        FavoritesList::iterator subIter = favorites->begin();
                         while (subIter != favorites->end())
                         {
                             if (!(*subIter)->isFolder && (*subIter)->parentFolder == (*iter)->name)
@@ -143,7 +143,7 @@ HTREEITEM PopulateLocationFolders(HWND hTree, CelestiaCore* appCore, HINSTANCE a
     // Associate the image list with the tree-view control.
     TreeView_SetImageList(hTree, himlIcons, TVSIL_NORMAL);
 
-    const FavoritesList* favorites = appCore->getFavorites();
+    FavoritesList* favorites = appCore->getFavorites();
     if (favorites != NULL)
     {
         //Create a subtree item called "Locations"
@@ -158,7 +158,7 @@ HTREEITEM PopulateLocationFolders(HWND hTree, CelestiaCore* appCore, HINSTANCE a
         tvis.item.iSelectedImage = 2;
         if (hParent = TreeView_InsertItem(hTree, &tvis))
         {
-            FavoritesList::const_iterator iter = favorites->begin();
+            FavoritesList::iterator iter = favorites->begin();
             while (iter != favorites->end())
             {
                 TVINSERTSTRUCT tvis;
@@ -188,81 +188,106 @@ HTREEITEM PopulateLocationFolders(HWND hTree, CelestiaCore* appCore, HINSTANCE a
     return hParent;
 }
 
-void BuildFavoritesMenu(HMENU menuBar, CelestiaCore* appCore)
+void BuildFavoritesMenu(HMENU menuBar, CelestiaCore* appCore, HINSTANCE appInstance, ODMenu* odMenu)
 {
     // Add favorites to locations menu
-    int numStaticItems = 2; //The number of items defined in the .rc file.
+    int numStaticItems = 2; // The number of items defined in the .rc file.
 
-    const FavoritesList* favorites = appCore->getFavorites();
+    // Ugly dependence on menu defined in celestia.rc; this needs to change
+    // if the locations menu is moved, or if another item is added to the
+    // menu bar.
+    // TODO: Fix this dependency
+    UINT locationsMenuPosition = 5;
+            
+
+    FavoritesList* favorites = appCore->getFavorites();
     if (favorites != NULL)
     {
         MENUITEMINFO menuInfo;
         menuInfo.cbSize = sizeof(MENUITEMINFO);
         menuInfo.fMask = MIIM_SUBMENU;
-        if (GetMenuItemInfo(menuBar, 4, TRUE, &menuInfo))
+        if (GetMenuItemInfo(menuBar, locationsMenuPosition, TRUE, &menuInfo))
         {
             HMENU locationsMenu = menuInfo.hSubMenu;
 
-            //First, tear down existing menu beyond separator.
-            while (DeleteMenu(locationsMenu, numStaticItems, MF_BYPOSITION));
+            // First, tear down existing menu beyond separator.
+            while (DeleteMenu(locationsMenu, numStaticItems, MF_BYPOSITION))
+                odMenu->DeleteItem(locationsMenu, numStaticItems);
 
-            //Don't continue if there are no items in favorites
+            // Don't continue if there are no items in favorites
             if (favorites->size() == 0)
                 return;
 
-            //Insert separator
+            // Insert separator
             menuInfo.cbSize = sizeof MENUITEMINFO;
             menuInfo.fMask = MIIM_TYPE | MIIM_STATE;
             menuInfo.fType = MFT_SEPARATOR;
             menuInfo.fState = MFS_UNHILITE;
-            InsertMenuItem(locationsMenu, numStaticItems++, TRUE, &menuInfo);
+            if (InsertMenuItem(locationsMenu, numStaticItems, TRUE, &menuInfo))
+            {
+                odMenu->AddItem(locationsMenu, numStaticItems);
+                numStaticItems++;
+            }
 
-            //Add folders and their sub items
+            // Add folders and their sub items
             int rootMenuIndex = numStaticItems;
-            int subMenuIndex, resourceIndex;
-            FavoritesList::const_iterator iter = favorites->begin();
+            int subMenuIndex, resourceIndex = 0;
+            FavoritesList::iterator iter = favorites->begin();
             while (iter != favorites->end())
             {
-                //Is this a folder?
+                // Is this a folder?
                 if ((*iter)->isFolder)
                 {
-                    //Create a submenu
+                    // Create a submenu
                     HMENU subMenu;
                     if (subMenu = CreatePopupMenu())
                     {
-                        //Create a menu item that displays a popup sub menu
+                        // Create a menu item that displays a popup sub menu
                         menuInfo.cbSize = sizeof MENUITEMINFO;
-                        menuInfo.fMask = MIIM_SUBMENU | MIIM_TYPE;
+                        menuInfo.fMask = MIIM_SUBMENU | MIIM_TYPE | MIIM_ID;
                         menuInfo.fType = MFT_STRING;
+                        menuInfo.wID = ID_LOCATIONS_FIRSTLOCATION + resourceIndex;
                         menuInfo.hSubMenu = subMenu;
                         menuInfo.dwTypeData = const_cast<char*>((*iter)->name.c_str());
                         if (InsertMenuItem(locationsMenu, rootMenuIndex, TRUE, &menuInfo))
                         {
-                            rootMenuIndex++;
+                            odMenu->AddItem(locationsMenu, rootMenuIndex);
+                            odMenu->SetItemImage(appInstance, menuInfo.wID, IDB_FOLDERCLOSED);
 
-                            //Now iterate through all Favorites and add items to this folder
-                            //where parentFolder == folderName
-                            resourceIndex = subMenuIndex = 0;
+                            rootMenuIndex++;
+                            resourceIndex++;
+
+                            // Now iterate through all Favorites and add items
+                            // to this folder where parentFolder == folderName
+                            subMenuIndex = 0;
                             string folderName = (*iter)->name;
-                            FavoritesList::const_iterator subIter = favorites->begin();
-                            while (subIter != favorites->end())
+                            iter++;
+                            while (iter != favorites->end())
                             {
-                                if (!(*subIter)->isFolder && (*subIter)->parentFolder == folderName)
+                                if (!(*iter)->isFolder && (*iter)->parentFolder == folderName)
                                 {
-                                    //Add items to sub menu
+                                    // Add items to sub menu
                                     menuInfo.cbSize = sizeof MENUITEMINFO;
                                     menuInfo.fMask = MIIM_TYPE | MIIM_ID;
                                     menuInfo.fType = MFT_STRING;
                                     menuInfo.wID = ID_LOCATIONS_FIRSTLOCATION + resourceIndex;
-                                    menuInfo.dwTypeData = const_cast<char*>((*subIter)->name.c_str());
-                                    InsertMenuItem(subMenu, subMenuIndex, TRUE, &menuInfo);
-                                    subMenuIndex++;
+                                    menuInfo.dwTypeData = const_cast<char*>((*iter)->name.c_str());
+                                    if(InsertMenuItem(subMenu, subMenuIndex, TRUE, &menuInfo))
+                                    {
+                                        odMenu->AddItem(subMenu, subMenuIndex);
+                                        odMenu->SetItemImage(appInstance, menuInfo.wID, IDB_LOCATION);
+                                        subMenuIndex++;
+                                    }
                                 }
-                                subIter++;
+                                else
+                                    break;
+
+                                iter++;
                                 resourceIndex++;
                             }
 
-                            //Add a disabled "(empty)" item if no items were added to sub menu
+                            // Add a disabled "(empty)" item if no items
+                            // were added to sub menu
                             if (subMenuIndex == 0)
                             {
                                 menuInfo.cbSize = sizeof MENUITEMINFO;
@@ -270,29 +295,36 @@ void BuildFavoritesMenu(HMENU menuBar, CelestiaCore* appCore)
                                 menuInfo.fType = MFT_STRING;
                                 menuInfo.fState = MFS_DISABLED;
                                 menuInfo.dwTypeData = "(empty)";
-                                InsertMenuItem(subMenu, subMenuIndex, TRUE, &menuInfo);
+                                if (InsertMenuItem(subMenu, subMenuIndex, TRUE, &menuInfo))
+                                {
+                                    odMenu->AddItem(subMenu, subMenuIndex);
+                                }
                             }
                         }
                     }
                 }
-                iter++;
+                else
+                    iter++;
             }
 
-            //Add root locations items
-            resourceIndex = 0;
+            // Add root locations items
             iter = favorites->begin();
             while (iter != favorites->end())
             {
-                //Is this a non folder item?
+                // Is this a non folder item?
                 if (!(*iter)->isFolder && (*iter)->parentFolder == "")
                 {
-                    //Append to locationsMenu
+                    // Append to locationsMenu
                     AppendMenu(locationsMenu, MF_STRING,
                         ID_LOCATIONS_FIRSTLOCATION + resourceIndex,
                         const_cast<char*>((*iter)->name.c_str()));
+
+                    odMenu->AddItem(locationsMenu, rootMenuIndex);
+                    odMenu->SetItemImage(appInstance, ID_LOCATIONS_FIRSTLOCATION + resourceIndex, IDB_LOCATION);
+                    rootMenuIndex++;
+                    resourceIndex++;
                 }
                 iter++;
-                resourceIndex++;
             }
         }
     }
@@ -344,8 +376,8 @@ void AddNewLocationFolderInTree(HWND hTree, char* folderName)
 
 void SyncTreeFoldersWithFavoriteFolders(HWND hTree, CelestiaCore* appCore)
 {
-    const FavoritesList* favorites = appCore->getFavorites();
-    FavoritesList::const_iterator iter;
+    FavoritesList* favorites = appCore->getFavorites();
+    FavoritesList::iterator iter;
     TVITEM tvItem;
     HTREEITEM hItem, hParent;
     char itemName[33];
@@ -398,7 +430,7 @@ void SyncTreeFoldersWithFavoriteFolders(HWND hTree, CelestiaCore* appCore)
                         //new folder must be inserted after the last item of the 
                         //last folder.
                         //Locate position of last folder.
-                        FavoritesList::const_iterator folderIter = favorites->begin();
+                        FavoritesList::iterator folderIter = favorites->begin();
                         iter = favorites->begin();
                         while (iter != favorites->end())
                         {
@@ -423,8 +455,8 @@ void SyncTreeFoldersWithFavoriteFolders(HWND hTree, CelestiaCore* appCore)
 
 void InsertLocationInFavorites(HWND hTree, char* name, CelestiaCore* appCore)
 {
-    const FavoritesList* favorites = appCore->getFavorites();
-    FavoritesList::const_iterator iter;
+    FavoritesList* favorites = appCore->getFavorites();
+    FavoritesList::iterator iter;
     TVITEM tvItem;
     HTREEITEM hItem;
     char itemName[33];
@@ -476,8 +508,8 @@ void InsertLocationInFavorites(HWND hTree, char* name, CelestiaCore* appCore)
 
 void DeleteLocationFromFavorites(HWND hTree, CelestiaCore* appCore)
 {
-    const FavoritesList* favorites = appCore->getFavorites();
-    FavoritesList::const_iterator iter;
+    FavoritesList* favorites = appCore->getFavorites();
+    FavoritesList::iterator iter;
     TVITEM tvItem;
     HTREEITEM hItem, hFolder;
     char itemName[33], folderName[33];
@@ -518,8 +550,8 @@ void DeleteLocationFromFavorites(HWND hTree, CelestiaCore* appCore)
                             {
                                 if ((*iter)->name == itemName || (*iter)->parentFolder == itemName)
                                 {
-                                    //Delete item from favorites.
-                                    ((FavoritesList*)favorites)->erase((FavoritesList::iterator)iter);
+				    // Delete item from favorites.
+                                    favorites->erase(iter);
                                 }
                                 else
                                     iter++;
@@ -534,11 +566,11 @@ void DeleteLocationFromFavorites(HWND hTree, CelestiaCore* appCore)
                         {
                             if ((*iter)->name == itemName && (*iter)->parentFolder == folderName)
                             {
-                                //Delete the tree item
+                                // Delete the tree item
                                 if (TreeView_DeleteItem(hTree, hItem))
                                 {
-                                    //Delete item from favorites.
-                                    ((FavoritesList*)favorites)->erase((FavoritesList::iterator)iter);
+                                    // Delete item from favorites.
+                                    favorites->erase(iter);
                                 }
                                 break;
                             }
@@ -553,8 +585,8 @@ void DeleteLocationFromFavorites(HWND hTree, CelestiaCore* appCore)
 
 void RenameLocationInFavorites(HWND hTree, char* newName, CelestiaCore* appCore)
 {
-    const FavoritesList* favorites = appCore->getFavorites();
-    FavoritesList::const_iterator iter;
+    FavoritesList* favorites = appCore->getFavorites();
+    FavoritesList::iterator iter;
     TVITEM tvItem;
     HTREEITEM hItem, hFolder;
     char itemName[33], folderName[33];
@@ -633,8 +665,8 @@ void RenameLocationInFavorites(HWND hTree, char* newName, CelestiaCore* appCore)
 
 void MoveLocationInFavorites(HWND hTree, CelestiaCore* appCore)
 {
-    const FavoritesList* favorites = appCore->getFavorites();
-    FavoritesList::const_iterator iter;
+    FavoritesList* favorites = appCore->getFavorites();
+    FavoritesList::iterator iter;
     TVITEM tvItem;
     TVINSERTSTRUCT tvis;
     HTREEITEM hDragItemFolder, hDropItem;
@@ -710,7 +742,7 @@ void MoveLocationInFavorites(HWND hTree, CelestiaCore* appCore)
         {
             if ((*iter)->name == dragItemName && (*iter)->parentFolder == dragItemFolderName)
             {
-                FavoritesList::const_iterator subIter;
+                FavoritesList::iterator subIter;
 
                 FavoritesEntry* fav = new FavoritesEntry();
                 fav->name = dragItemName;
@@ -738,8 +770,8 @@ void MoveLocationInFavorites(HWND hTree, CelestiaCore* appCore)
                                   (*subIter)->parentFolder != "")
                                   subIter++;
 
-                            //Insert new location at position in iteration.
-                            ((FavoritesList*)favorites)->insert((FavoritesList::iterator)subIter, fav);
+                            // Insert new location at position in iteration.
+                            favorites->insert(subIter, fav);
                             break;
                         }
 
@@ -752,8 +784,8 @@ void MoveLocationInFavorites(HWND hTree, CelestiaCore* appCore)
                     {
                         if ((*iter)->name == dragItemName && (*iter)->parentFolder == dragItemFolderName)
                         {
-                            //Delete item from favorites.
-                            ((FavoritesList*)favorites)->erase((FavoritesList::iterator)iter);
+                            // Delete item from favorites.
+                            favorites->erase(iter);
                             break;
                         }
 
@@ -762,11 +794,11 @@ void MoveLocationInFavorites(HWND hTree, CelestiaCore* appCore)
                 }
                 else
                 {
-                    //Just append item to end for root folders
-                    ((FavoritesList*)favorites)->insert((FavoritesList::iterator)favorites->end(), fav);
+                    // Just append item to end for root folders
+                    favorites->insert(favorites->end(), fav);
 
-                    //Delete item from favorites.
-                    ((FavoritesList*)favorites)->erase((FavoritesList::iterator)iter);
+                    // Delete item from favorites.
+                    favorites->erase(iter);
                 }
 
                 break;
@@ -971,4 +1003,4 @@ HTREEITEM GetTreeViewItemHandle(HWND hTree, char* path, HTREEITEM hParent)
 
     return hItem;
 }
-*/
\ No newline at end of file
+*/
