@@ -9,6 +9,8 @@
 
 #include "color.h"
 
+const Color Color::White = Color(1.0f, 1.0f, 1.0f);
+const Color Color::Black = Color(0.0f, 0.0f, 0.0f);
 
 template<class T> T clamp(T x)
 {
