@@ -24,6 +24,10 @@ enum ContentType {
     Content_AVI            = 9,
     Content_CelestiaCatalog = 10,
     Content_DDS             = 11,
+    Content_CelestiaStarCatalog = 12,
+    Content_CelestiaDeepSkyCatalog = 13,
+    Content_CelestiaScript         = 14,
+    Content_CelestiaLegacyScript   = 15,
     Content_Unknown         = -1,
 };
 
